#!/usr/bin/env python
# Copyright 2016 Cumulus Networks, Inc
#
# ecmpcalc utility for the mlx platform

import subprocess
import sys
import os
import re
import ipaddr
import cumulus.porttab
from cumulus.ecmpcalc_util import asic2linux, asictrunkmbr2linux
try:
    from cumulus.mlx import mlx_open_connection
    from cumulus.mlx import mlx_close_connection
    from cumulus.mlx import mlx_get_uc_route
    from cumulus.mlx import mlx_get_ecmp
    from cumulus.mlx import mlx_get_operational_ecmp
    from cumulus.mlx import mlx_get_neighbor
    from cumulus.mlx import mlx_get_interface
    from cumulus.mlx import mlx_get_fdb
    from cumulus.mlx import mlx_get_lag_member_ids
    from cumulus.mlx import mlx_get_vport_port
    from cumulus.mlx import mlx_get_reg_recr_v2
    from cumulus.mlx import mlx_invalid_port_id

except ImportError:
    raise NotImplementedError


class RouteInfo:
    def __init__(self, vrf, dst, verbose=False):
        self.verbose = verbose
        self.vrf = vrf
        self.dst = dst
        self.isRoutable = False
        self.isEcmp = False
        self.oper_nhops = []
        self.nhops = []
        self._routeLkup()

    def _routeLkup(self):
        dstAddr = ipaddr.IPAddress(self.dst.addrstr)

        # LPM on dstAddr
        rtTable = mlx_get_uc_route(version=dstAddr.version, vrid=None)
        #if self.verbose:
        #    sys.stderr.write('rtTable:\n  %s\n' % rtTable)
        net = 0
        ecmp_id = -1
        """
        Rt format:
        ('0 18.0.0.0/16', {'action': 'FORWARD', 'vrid': 0, 'prio': 0, 'next_hop_cnt': 3, 'erif': 6L, 'ecmp_id': 6, 'next_hops': ['11.0.0.25', '11.0.0.29', '11.0.0.37'], 'type': 'NEXT_HOP'})
        """
        for rt in rtTable.items():
            prefix = rt[0].split(' ')[1]
            newNet = ipaddr.IPNetwork(prefix)
            newEcmpId = rt[1].get('ecmp_id')
            if dstAddr in newNet and (ecmp_id == -1 or newNet > net):
                net = newNet
                ecmp_id = newEcmpId
                self.nhops = rt[1].get('next_hops', [])
                if self.verbose:
                    sys.stderr.write('matching rt: %s: %s\n' % (rt[0], rt[1]))
        if self.verbose:
            sys.stderr.write('located net %s, ecmp_id %d\n' % (net, ecmp_id))
        if ecmp_id != -1:
            self.isRoutable = True
        
        # Locate ecmp group
        if ecmp_id != -1 and len(self.nhops) > 1:
            ecmpTable = mlx_get_operational_ecmp(ecmp_id)
            #if self.verbose:
            #    sys.stderr.write('ecmpTable:\n  %s\n' % ecmpTable)
            if ecmpTable:
                self.oper_nhops = ecmpTable.get(ecmp_id, [])
            if len(self.oper_nhops):
                self.isEcmp = True

        #if self.verbose:
        #    sys.stderr.write('ecmpgrp %s, %s\n' % (self.nhops, self.oper_nhops))

    def getL3PktRes(self):
        return self.isRoutable, self.isEcmp

    def getNextHopCnt(self):
        return len(self.oper_nhops)

    def _getLagMemberFromLagLogicalId(self, lport):
        lagMembers = mlx_get_lag_member_ids(swid=0, logical_bond_id=lport)
        if not lagMembers:
            raise RuntimeError('No lag members in lag %d', lport)
        if self.verbose:
            sys.stderr.write('found lagMembers:  %s\n' % (lagMembers))

        # xlate logical port id to phy port id
        return (lagMembers[0] >> 8) & 0xff

    def _logical_port_to_linux_interface(self, logical_port):
        porttab = cumulus.porttab.porttab()
        chip_interface = 'xe%d' % ((logical_port - (1 << 16)) >> 8) # XXX assumes the unit is 0: s.b. handled by portmap
        linux_interface = porttab.bcm2linux(chip_interface)
        return linux_interface

    def _getLinuxIntfFromNhop(self, nextHopIndex):
        # use nextHopIndex to get neigh entry
        dstAddr = ipaddr.IPAddress(self.dst.addrstr)
        nh_entry = self.oper_nhops[nextHopIndex]
        if nh_entry['type'] != 'IP':
            raise RuntimeError('non-IP neigh entry type not yet supported')

        nhop = nh_entry['address']
        rif  = nh_entry['rif']

        # use rif to get bridge_id
        l3Intfs = mlx_get_interface().get('entries')
        #if self.verbose:
        #    sys.stderr.write('l3Intfs:\n  %s\n' % l3Intfs.get('entries'))
        bridge_id = -1
        # l3Intf format
        # 36 {'vlan': None, 'egress mode': None, 'swid': 0L, 'vrid': 0L, 'mtu': 1500L, 'mac': '44:38:39:00:a4:00', 'mac_addr': None, 'bridge id': 4133L, 'type': 'bridge', 'port': None}
        for l3Intf in l3Intfs.items():
            if rif == l3Intf[0]:
                if self.verbose:
                    sys.stderr.write('found l3Intf:  %s %s\n' % (l3Intf[0], l3Intf[1]))
                bridge_id = l3Intf[1].get('bridge id')
                vport = l3Intf[1].get('vport')
                rif_type = l3Intf[1].get('type')
                break
        if bridge_id == -1:
            raise RuntimeError('No l3 interface for %d', rif)

        # forwarding to rif of type vport does not involve any dmac lookup
        if rif_type == 'vport':
            # use the vport ID to get the linux interface
            logical_port = mlx_get_vport_port(vport)
            print 'vport %d logical port %d' % (vport, logical_port),
            linuxIntf = self._logical_port_to_linux_interface(logical_port)
            print 'linux interface %s' % linuxIntf
        else:
            neighTable = mlx_get_neighbor(dstAddr.version, rif)
            #if self.verbose:
            #    sys.stderr.write('neighTable:\n  %s\n' % neighTable)
            dmac = ""
            # neigh format
            # 11.0.0.6 36 {'action': 'FORWARD', 'rif': 36L, 'mac': '00:02:00:00:00:06', 'vrid': 36L, 'prio': 0}
            for neigh in neighTable.items():
                if neigh[0].split()[0] == nhop:
                    if self.verbose:
                        sys.stderr.write('found neigh:  %s %s\n' % (neigh[0], neigh[1]))
                    dmac = neigh[1].get('mac')
                    break
            if not dmac:
                raise RuntimeError('No neigh entry for %s %d', nhop, rif)

            # use dmac from neigh and bridge_id to get the linux interface
            fdb = mlx_get_fdb(swid = 0)
            #if self.verbose:
            #    sys.stderr.write('fdb:\n  %s\n' % fdb)
            # fdb entry format
            # 00:02:00:00:00:06 {'fid_vid': 4133L, 'swid': 0, 'logical_port': 811860224, 'lport_type': 3, 'entry_type': 'UC_AGEABLE', 'action': 'FWD', 'port_id': '1', 'vlan_id': 100}
            port = -1
            logical_port = -1
            port_type = 0
            for mac in fdb:
                if mac != dmac:
                    continue
                for fdb_info in fdb[mac]:
                    if fdb_info.get('fid_vid') == bridge_id and\
                        fdb_info.get('entry_type') != 'UC_AGED':
                        logical_port = fdb_info.get('logical_port')
                        port = int(fdb_info.get('port_id'))
                        port_type = fdb_info.get('lport_type')
                        if self.verbose:
                            sys.stderr.write('found fdb entry:  %s %s\n' % (mac, fdb_info))
                        break
            if port == -1:
                raise RuntimeError('No fdb entry for %d %s', bridge_id, dmac)

            if port_type & 1:
                # LAG
                if self.verbose:
                    sys.stderr.write('lag_id: %d/%d\n' % (port, logical_port))
                port = self._getLagMemberFromLagLogicalId(logical_port)
                if self.verbose:
                    sys.stderr.write('lag member: %d\n' % (port))
                linuxIntf = asictrunkmbr2linux('xe%d' % port)
            elif port_type & 2:
                # phy port
                if self.verbose:
                    sys.stderr.write('phy_port: %d/%d\n' % (port, logical_port))
                linuxIntf = asic2linux('xe%d' % port)
            else:
                raise RuntimeError('logical port %d:%d has an unexpected type', port_type, port)
        return linuxIntf

    def getLinuxIntf(self, nextHopIndex):
        nextHopCnt = self.getNextHopCnt()
        if nextHopIndex >= nextHopCnt:
            raise RuntimeError('Error: traffic to %s egress index %d out of range %d',
                               self.dst, nextHopIndex, nextHopCnt)
        if self.verbose:
            sys.stderr.write('traffic to %s egress index %d range $d\n' % (self.dst,
                             nextHopIndex, nextHopCnt))

        return self._getLinuxIntfFromNhop(nextHopIndex)
        #return 'rif: %d' % rif


def reg_value_to_4tuple_str(num, width):
    if width == 16:
        return """0x%02x 0x%02x 0x%02x 0x%02x\n""" % (
               0, 0,
               ((num >> (width - 8)) & 0xff), ((num >> (width - 16)) & 0xff))
    if width == 32 or width == 64:
        return """0x%02x 0x%02x 0x%02x 0x%02x\n """ % (
               ((num >> (width -  8)) & 0xff), ((num >> (width - 16)) & 0xff),  
               ((num >> (width - 24)) & 0xff), ((num >> (width - 32)) & 0xff))
    raise RuntimeError('only width 16, 32, or 64 supported')

class HashCalc:
    _mlxHashCalcPath = '/usr/lib/cumulus/hash_calculator'
    _mlxPath = '/var/run/mlx_ecmpcalc'
    _mlxEpPath = _mlxPath + '/ep'
    _mlxPktPath = _mlxPath + '/pkt'

    def __init__(self, inIntf, frame, numNextHops, verbose=False):
        self.verbose = verbose
        self.inIntf = inIntf
        self.frame = frame

        # hardcoded hash_calc params
        self.param_et = 0x8025 # REG_ID_RECRV2
        self.param_r = 1  # ???

        try:
            os.makedirs(self._mlxPath)
        except OSError:
            pass

        # Get the REG_ID_RECRV2 value as programmed by the SDK.
        recrv2 = mlx_get_reg_recr_v2(mlx_invalid_port_id())

        # struct ku_recr_v2_reg has the following C definition
        #
        # struct ku_recr_v2_reg {
        #     uint8_t                     per_port_configuration;
        #     uint8_t                     local_port;
        #     uint8_t                     symmetric_hash;
        #     sxd_router_ecmp_hash_type_t hash_type;
        #     uint32_t                    seed;
        #     uint32_t                    general_fields;
        #     uint16_t                    outer_header_enables;
        #     uint32_t                    outer_header_field_enables[5];
        #     uint16_t                    inner_header_enables;
        #     uint64_t                    inner_header_field_enables;
        # };
        #
        # However, the emad file expects the data *packed* as follows:
        #
        # struct ku_recr_v2_reg {
        #     uint8_t                     per_port_configuration;
        #     uint8_t                     local_port;
        #     uint8_t                     symmetric_hash;
        #     uint8_t                     hash_type;
        #     uint32_t                    reserved_0;
        #     uint32_t                    seed;
        #     uint32_t                    general_fields;
        #     uint16_t                    reserved_1;
        #     uint16_t                    outer_header_enables;
        #     uint32_t                    outer_header_field_enables[5];
        #     uint32_t                    reserved_2;
        #     uint16_t                    reserved_3;
        #     uint16_t                    inner_header_enables;
        #     uint64_t                    inner_header_field_enables;
        # } __attribute__((__packed__));
        #
        recrv2_str  = """0x%02x 0x%02x 0x%02x 0x%02x
                      """ % (recrv2['per_port_configuration'], recrv2['local_port'],
                             recrv2['symmetric_hash'],         recrv2['hash_type'])
        # reserved_0
        recrv2_str += reg_value_to_4tuple_str(0, 32)
        recrv2_str += reg_value_to_4tuple_str(recrv2['seed'], 32)
        recrv2_str += reg_value_to_4tuple_str(recrv2['general_fields'], 32)
        # reserved_1 and outer_header_enables
        recrv2_str += reg_value_to_4tuple_str(recrv2['outer_header_enables'], 16)
        recrv2_str += reg_value_to_4tuple_str(recrv2['outer_header_field_enables0'], 32)
        recrv2_str += reg_value_to_4tuple_str(recrv2['outer_header_field_enables1'], 32)
        recrv2_str += reg_value_to_4tuple_str(recrv2['outer_header_field_enables2'], 32)
        recrv2_str += reg_value_to_4tuple_str(recrv2['outer_header_field_enables3'], 32)
        recrv2_str += reg_value_to_4tuple_str(recrv2['outer_header_field_enables4'], 32)
        # reserved_2
        recrv2_str += reg_value_to_4tuple_str(0, 32)

        # As of 04/27/17, SDK 4.2.4000, HashCalc v1.2.0 is known to have
        # issue when inner_header_enables are enabled. Since we have not started
        # to validate hashing with tunneled traffic yet, we can work around this
        # issue by telling HashCalc that inner_header_enables are disabled.
        # 
        # reserved_3 and inner_header_enables
        #recrv2_str += reg_value_to_4tuple_str(recrv2['inner_header_enables'], 16)
        recrv2_str += reg_value_to_4tuple_str(0, 32)
        recrv2_str += reg_value_to_4tuple_str(recrv2['inner_header_field_enables'], 64)
        recrv2_str += reg_value_to_4tuple_str(recrv2['inner_header_field_enables'], 32)

        with open(self._mlxEpPath, 'w') as file:
            file.write(recrv2_str)

        self.nextHopIndex = -1
        self._runCalc(numNextHops)

    def _getPortMac(self):
        _macPath = '/sys/class/net/%s/address' % self.inIntf
        if not os.path.isfile(_macPath):
            raise RuntimeError('%s does not exist' % _macPath)
        try:
            with open(_macPath) as f:
                pmac = f.readline().strip()
        except IOError:
            raise RuntimeError('Contents of %s are not accessible' % _macPath)

        return pmac

    def _morphPkt(self, pkt):
        # in-packet is a hex-dump; out-pkt needs to be comma-space
        # separated hex-bytes
        pktList = [pkt[i:i+2] for i in range(0, len(pkt), 2)]
        if len(pktList):
            pktList[0] = '0x' + pktList[0]
        return ', 0x'.join(pktList)

    def _buildPkt(self):
        # we need to build a routable ethernet header; to do that -
        # 1. vlan has to be valid i.e. source port has to be a member of the
        #    vlan. Best guess would be PVID.
        # 2. dmac has to be a mystation entry. Best guess would again be the
        #    port mac.
        pmac = self._getPortMac()
        vid = self.frame.vid.value if self.frame.vid else 0
        pkt = self.frame.buildVisibilityPkt(smac=pmac, dmac=pmac, vid=vid)

        if not pkt:
            raise RuntimeError('Error: visibility pkt trace generation failed')

        # convert the hexdump into some thing that mlx hash_calc recogonizes
        pkt = self._morphPkt(pkt)
        with open(self._mlxPktPath, 'w') as file:
            file.write(pkt)

        if self.verbose:
            sys.stderr.write('Trace pkt:\n  %s\n' % pkt)

    def _runCalc(self, numNextHops):
        if not os.path.isfile(self._mlxHashCalcPath):
            raise RuntimeError('Cannot find %s' % self._mlxHashCalcPath)

        # build and write pkt to _mlxPktPath
        self._buildPkt()

        # hash_calc
        cmdStr = '%s -p %s -ep %s -s %d -o %d -et 0x%x -r %d' %\
                    (self._mlxHashCalcPath, self._mlxPktPath, 
                     self._mlxEpPath, self.frame.sportid.value, numNextHops,
                     self.param_et, self.param_r)
        if self.verbose:
            sys.stderr.write('run %s\n' % cmdStr)
        try:
            calcOut = subprocess.check_output(cmdStr.split())
        except subprocess.CalledProcessError:
            calcOut = ""
        if self.verbose:
            sys.stderr.write('calcOut:\n  %s\n' % calcOut)

        # get nexthop index
        pat = re.compile('from \d+ we picked the index port (?P<index>\d+)')
        obj = pat.search(calcOut)
        if obj:
            self.nextHopIndex = int(obj.group('index'))

        if self.verbose:
            sys.stderr.write('nexthopIndex %d\n' % self.nextHopIndex)

    def getNextHopIndex(self):
        return self.nextHopIndex


def getEcmpMbrLinuxIntf(inIntf, frame, verbose=False):
    mlx_open_connection()

    # LPM
    # XXX - need to add vrf support to ecmpcalc
    vrf = 0 
    if verbose:
        sys.stderr.write('Run ecmpcalc %s\n' % frame.dst)
    routeInfo = RouteInfo(vrf, frame.dst, verbose)
    isRoutable, isEcmp = routeInfo.getL3PktRes()
    if not isRoutable:
        raise RuntimeError('Error: traffic to %s will not be L3 switched' % frame.dst)
    if not isEcmp:
        raise RuntimeError('Error: traffic to %s will not ECMP' % frame.dst)
    numNextHops = routeInfo.getNextHopCnt()

    # nextHopIndex for a ecmp group
    hashCalc = HashCalc(inIntf, frame, numNextHops, verbose)
    nextHopIndex = hashCalc.getNextHopIndex()
    if nextHopIndex == -1:
        raise RuntimeError('hash_calc failed to locate a next hop index')

    # swpX associated with nexthop
    outIntf = routeInfo.getLinuxIntf(nextHopIndex)

    mlx_close_connection()
    return outIntf
