#
# Copyright (c) 2019 Cumulus Networks, Inc.  All Rights Reserved.
#

import os
import subprocess
import syslog
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

# 10/100/1000 port LEDs are driven by the BCM54282 PHYs.  The PHY uses the
# LED[2] pins for Link/Activity and LED[0] pins for speed indication.

class AcctonAS4222PhyGigE(cumulus.platform.Hurricane21GPort):
    def __init__(self, label, hw_port, phy_addr, prim_offset):
        cumulus.platform.Hurricane21GPort.__init__(self,
                                               label=label,
                                               hw_port=hw_port,
                                               phy_type="50282",
                                               phy_addr=phy_addr,
                                               phy_layer="1GBaseT",
                                               prim_offset=prim_offset,
                                               leds=None,
                                               unit=0)

class AcctonAS4222PhylessGigE(cumulus.platform.Hurricane21GPort):
    _set_led = '''
    ;;; AcctonAS4222PhylessGigE

set_led:
        port a
        pushst  PORTSTATUS_ZERO
        pack

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ZERO
        pack

        ret
'''

    def __init__(self, label, hw_port, prim_offset, leds):
        cumulus.platform.Hurricane21GPort.__init__(self,
                                               label=label,
                                               hw_port=hw_port,
                                               phy_type=None,
                                               phy_addr=None,
                                               phy_layer="1GBaseT",
                                               prim_offset=prim_offset,
                                               leds=leds,
                                               unit=0)

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.hw_port_num(mode, 0))
        for led in led_index_list:
            set_calls[led] = call
            set_calls[led + 1] = None
            set_calls[led + 2] = None
            set_calls[led + 3] = None

        return set_calls

class AcctonAS4222SFPp(cumulus.platform.Hurricane210GPort):
    '''
;;; Accton AS4222-28PE
;;;
;;; There are two LEDs for each of the four SFP+ ports.  The two bits of
;;; LED data operate as follows (the first bit lights the right LED and
;;; the second bit lights the left LED):
;;;
;;;       2nd bit       1st bit
;;;      +-------+     +-------+
;;;    0 | green |     | green |
;;;      +-------+     +-------+
;;;    1 |  off  |     |  off  |
;;;      +-------+     +-------+
;;;
;;; The first bit will be used to indicate Link, and the second bit will
;;; be used it indicate activity.
;;;
;;; '''
    _set_led = '''
set_led:
        port a
        pushst  PORTSTATUS_ZERO
        pack

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ZERO
        pack

        ret
'''
    def __init__(self, label, hw_port, leds, rx_polarity_flip=None,
                 tx_polarity_flip=None, tx_lane_map=None, rx_lane_map=None):
        cumulus.platform.Hurricane210GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type=None,
                                                phy_addr=None,
                                                phy_layer="SFP+",
                                                leds=leds,
                                                rx_lane_map=rx_lane_map,
                                                tx_lane_map=tx_lane_map,
                                                rx_polarity_flip=rx_polarity_flip,
                                                tx_polarity_flip=tx_polarity_flip,
                                                unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    # override led_port_num because this platform has a single Hurricane2 and no Higig ports 
    def led_port_num(self, mode, sub):
        num = self.hw_intf_num(mode, sub)
        prefix = self.hw_intf_prefix(mode, sub)
        if prefix == 'ge':
            return num + 2
        elif prefix == 'xe':
            return num + 26
        else:
            raise RuntimeError('bad interface prefix')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.led_port_num(mode, 0))

        for led in led_index_list:
            set_calls[led] = call
            set_calls[led + 1] = None
            set_calls[led + 2] = None
            set_calls[led + 3] = None

        return set_calls

class AcctonAS4222LED(cumulus.ledup.Hurricane2LEDController):
    num_leds = 96

# Switch Ports

class AcctonAS4222Switch(cumulus.platform.BCMSwitch):
    ledup = AcctonAS4222LED
    _phy_gige = AcctonAS4222PhyGigE
    _phyless_gige = AcctonAS4222PhylessGigE
    _sfp = AcctonAS4222SFPp
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _phy_gige(label="1", hw_port='ge16', phy_addr=0x01, prim_offset=0x1200),
        _phy_gige(label="2", hw_port='ge17', phy_addr=0x02, prim_offset=0x1201),
        _phy_gige(label="3", hw_port='ge18', phy_addr=0x03, prim_offset=0x1202),
        _phy_gige(label="4", hw_port='ge19', phy_addr=0x04, prim_offset=0x1203),
        _phy_gige(label="5", hw_port='ge20', phy_addr=0x05, prim_offset=0x1204),
        _phy_gige(label="6", hw_port='ge21', phy_addr=0x06, prim_offset=0x1205),
        _phy_gige(label="7", hw_port='ge22', phy_addr=0x07, prim_offset=0x1206),
        _phy_gige(label="8", hw_port='ge23', phy_addr=0x08, prim_offset=0x1207),

        _phyless_gige(label="9",  hw_port='ge0',  prim_offset=0x0200, leds=(0, 28)),
        _phyless_gige(label="10", hw_port='ge1',  prim_offset=0x0201, leds=(0, 24)),
        _phyless_gige(label="11", hw_port='ge2',  prim_offset=0x0202, leds=(0, 20)),
        _phyless_gige(label="12", hw_port='ge3',  prim_offset=0x0203, leds=(0, 16)),
        _phyless_gige(label="13", hw_port='ge4',  prim_offset=0x0204, leds=(0, 12)),
        _phyless_gige(label="14", hw_port='ge5',  prim_offset=0x0205, leds=(0,  8)),
        _phyless_gige(label="15", hw_port='ge6',  prim_offset=0x0206, leds=(0,  4)),
        _phyless_gige(label="16", hw_port='ge7',  prim_offset=0x0207, leds=(0,  0)),

        _phyless_gige(label="17", hw_port='ge8',  prim_offset=0x0a00, leds=(0, 32)),
        _phyless_gige(label="18", hw_port='ge9',  prim_offset=0x0a01, leds=(0, 36)),
        _phyless_gige(label="19", hw_port='ge10', prim_offset=0x0a02, leds=(0, 40)),
        _phyless_gige(label="20", hw_port='ge11', prim_offset=0x0a03, leds=(0, 44)),
        _phyless_gige(label="21", hw_port='ge12', prim_offset=0x0a04, leds=(0, 48)),
        _phyless_gige(label="22", hw_port='ge13', prim_offset=0x0a05, leds=(0, 52)),
        _phyless_gige(label="23", hw_port='ge14', prim_offset=0x0a06, leds=(0, 56)),
        _phyless_gige(label="24", hw_port='ge15', prim_offset=0x0a07, leds=(0, 60)),

        _sfp(label="25", hw_port='xe0', leds=(0, 80), tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="26", hw_port='xe1', leds=(0, 84), tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="27", hw_port='xe2', leds=(0, 88), tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="28", hw_port='xe3', leds=(0, 92), tx_polarity_flip=False, rx_polarity_flip=False,),
    )

class AcctonAS4222CPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/pci0000:00/0000:00:12.0/i2c-1/i2c-11/11-0060'

# Intel CPU temp sensors

class AcctonAS4222CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 65
    temp_max = 71
    temp_crit = 91

class AcctonAS4222CPUTemp1(AcctonAS4222CoreTemp):
    name = 'Temp1'
    description = 'Physical id 0'
    driver_hwmon = ['temp1']

class AcctonAS4222CPUTemp2(AcctonAS4222CoreTemp):
    name = 'Temp2'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class AcctonAS4222CPUTemp3(AcctonAS4222CoreTemp):
    name = 'Temp3'
    description = 'Core 6'
    driver_hwmon = ['temp8']

class AcctonAS4222CPUTemp4(AcctonAS4222CoreTemp):
    name = 'Temp4'
    description = 'Core 8'
    driver_hwmon = ['temp10']

class AcctonAS4222CPUTemp5(AcctonAS4222CoreTemp):
    name = 'Temp5'
    description = 'Core 12'
    driver_hwmon = ['temp14']

# Hurricane2 internal temp sensors

class AcctonAS4222Hurricane2Temp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'

class AcctonAS4222Hurricane2Temp1(AcctonAS4222Hurricane2Temp):
    name = 'Temp6'
    driver_hwmon = ['temp1']

# Board temp sensors

class AcctonAS4222Temp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']
    temp_max = 70
    temp_fan_max = 78
    temp_crit = 85

class AcctonAS4222Temp1(AcctonAS4222Temp):
    driver_path = '/sys/bus/i2c/devices/25-0048'
    name = 'Temp7'
    description = 'Ambient 1'

class AcctonAS4222Temp2(AcctonAS4222Temp):
    driver_path = '/sys/bus/i2c/devices/25-0049'
    name = 'Temp8'
    description = 'Ambient 2'

class AcctonAS4222Temp3(AcctonAS4222Temp):
    driver_path = '/sys/bus/i2c/devices/25-004a'
    name = 'Temp9'
    description = 'Fan board'

class AcctonAS4222Temp4(AcctonAS4222Temp):
    driver_path = '/sys/bus/i2c/devices/25-004b'
    name = 'Temp10'
    description = 'CPU board'

# Fans

class AcctonAS4222Fan(cumulus.sensors.Fan_Unit):
    cpld_path = AcctonAS4222CPLD.path
    driver_path = cpld_path
    pwm_hwmon = ['fan_pwm']
    minpwm = 1
    maxpwm = 8

class AcctonAS4222Fan1(AcctonAS4222Fan):
    name = "Fan1"
    description = "Fan 1"
    cpld_hwmon = ['fan1']

class AcctonAS4222Fan2(AcctonAS4222Fan):
    name = "Fan2"
    description = "Fan 2"
    cpld_hwmon = ['fan2']

class AcctonAS4222Fan3(AcctonAS4222Fan):
    name = "Fan3"
    description = "Fan 3"
    cpld_hwmon = ['fan3']

# PSU

class AcctonAS4222PSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    description = "Internal PSU"
    cpld_path = AcctonAS4222CPLD.path
    cpld_hwmon = ['psu_pwr']

# LEDs
'''
   Accton AS4222-28PE LEDs

   System:
       green           - all systems OK
       amber blinking  - service required
   PSU:
       green           - power OK
       amber blinking  - power supply failure
   Fan:
       green           - fans OK
       amber blinking  - fan failure
   POE:
       green           - POE OK
       amber blinking  - POE failure
'''
class AcctonAS4222LED(cumulus.sensors.StatusLED_Unit):
    led_path = AcctonAS4222CPLD.path

class AcctonAS4222SystemLED(AcctonAS4222LED):
    name = 'System LED'
    led_location = 'led_system'
    good_led_color = 'green'
    bad_led_color = 'amber_blinking'

class AcctonAS4222PSULED(AcctonAS4222LED):
    name = 'PSU LED'
    led_location = 'led_psu'
    sources = ['PSU1']
    good_led_color = 'green'
    bad_led_color = 'amber_blinking'

class AcctonAS4222FanLED(AcctonAS4222LED):
    name = 'Fan LED'
    led_location = 'led_fan'
    sources = ['Fan1', 'Fan2', 'Fan3' ]
    good_led_color = 'green'
    bad_led_color = 'amber_blinking'

class AcctonAS4222POELED(AcctonAS4222LED):
    name = 'POE LED'
    led_location = 'led_poe'
    good_led_color = 'green'
    bad_led_color = 'off'
    cpld_path = AcctonAS4222CPLD.path
    present = 'poe_present'
    enable = 'poe_enable'

    def get_cpld_value(self, source):
        try:
            path = os.path.join(self.cpld_path, source)
            value = open(path, 'r').read()
        except:
            self.update_msg('Unable to read CPLD value: %s' % path)
            return None
        return value.strip()

    def set_cpld_value(self, source, value):
        path = os.path.join(self.cpld_path, source)
        try:
            fh = open(path, 'w+')
            x = fh.write(value)
            fh.close()
        except:
            self.update_msg('Unable to update CPLD register: %s' %source)
            return None
        return x

    def led_update(self, color):
        if ((self.get_cpld_value(self.present) == '1') and
            (self.get_cpld_value(self.enable) == '1')):
            self.set_cpld_value(self.led_location, self.good_led_color)
        else:
            self.set_cpld_value(self.led_location, self.bad_led_color)

# PWM

class AcctonAS4222PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (AcctonAS4222Fan1(),
            AcctonAS4222Fan2(),
            AcctonAS4222Fan3(),
            )
    temps = (AcctonAS4222CPUTemp1(),
             AcctonAS4222CPUTemp2(),
             AcctonAS4222CPUTemp3(),
             AcctonAS4222CPUTemp4(),
             AcctonAS4222CPUTemp5(),
             AcctonAS4222Hurricane2Temp1(),
             AcctonAS4222Temp1(),
             AcctonAS4222Temp2(),
             AcctonAS4222Temp3(),
             AcctonAS4222Temp4(),
             )

# Units

class AcctonAS4222LEDs():
    units = (AcctonAS4222SystemLED(),
             AcctonAS4222PSULED(),
             AcctonAS4222FanLED(),
             AcctonAS4222POELED(),
             )

class AcctonAS4222Sensors():
    units = (AcctonAS4222CPUTemp1(),
             AcctonAS4222CPUTemp2(),
             AcctonAS4222CPUTemp3(),
             AcctonAS4222CPUTemp4(),
             AcctonAS4222CPUTemp5(),
             AcctonAS4222Hurricane2Temp1(),
             AcctonAS4222Temp1(),
             AcctonAS4222Temp2(),
             AcctonAS4222Temp3(),
             AcctonAS4222Temp4(),
             AcctonAS4222Fan1(),
             AcctonAS4222Fan2(),
             AcctonAS4222Fan3(),
             AcctonAS4222PSU1(),
             )

class AcctonAS4222PWM():
    units = (AcctonAS4222PWM1(),
             )

class AcctonAS4222(cumulus.platform.Platform):
    name = 'accton,as4222_28pe'
    chip = cumulus.platform.Hurricane2Chip()
    switch = AcctonAS4222Switch(chip)
    cpld = AcctonAS4222CPLD()
    statusleds = AcctonAS4222LEDs()
    pwms = AcctonAS4222PWM()
    sensors = AcctonAS4222Sensors()

Platform = AcctonAS4222
