#
# Copyright 2016, 2017.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors


class AcctonAS7712QSFP28(cumulus.platform.Tomahawk100GPort):
    '''
    7712 has 14 bits of LED data controlling 4 LEDs per port.
    The 14 bits contain 2 bits of Lane number and speed which
    which in combination decide what color will be displayed. 
    The bit stream looks like below and starts with bits for
    lane1 all the way to lane4.

    Lane[1:0] speed[1:0] linken | speed[1:0] linken | 
    speed[1:0] linken | speed[1:0] linken  

    A speed of 00 and lane of 00 turns on green for a
    paritcular lane.
    '''
    _set_led_common = '''
set_off_led:
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ZERO
        pack
        ret

set_port_led:
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ZERO
        pack


        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ZERO
        pack
check_and_pack:
        ld      b, (ID_ENABLE_P)
        tst     b, ID_ENABLE_BIT
        jc      id_mode
        port    a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack
        ret
id_mode:
        push    0
        cmp     a, (ID_PORT_P)
        jnz     id_pack
        ld      b, (WIGGLE_STATE_P)
        and     b, (WAG_STATE_P)
        push    b
id_pack:
        pack
        ret
'''
    _set_led_1x = '''
;;;
;;; In 1x mode, only the green color of the first LED is used to
;;; indicate link/activity. The rest are off.
;;;

set_led:
        call set_port_led
        ; setting other bits to zero
        call set_off_led
        call set_off_led
        call set_off_led
        ret
'''

    _set_led_2x = '''
;;;
;;; In 2x mode, we use the green color of the first and second LED to
;;; indicate link/activity. The rest are off.
;;;
set_2x_led:
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ZERO
        pack
        inc     a
        inc     a
        jmp     check_and_pack


set_led:
        call set_port_led
        call set_2x_led
        call set_off_led
        call set_off_led
        ret
'''

    _set_led_4x = '''
;;;
;;; In 4x mode, the green color of all four LEDs are used to
;;; indicate link/activity.
;;;
set_4x_led:
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ZERO
        pack

        inc     a
        jmp     check_and_pack

set_led:
        call set_port_led
        call set_4x_led
        call set_4x_led
        call set_4x_led
        ret
'''

    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
         tx_polarity_flip, rx_polarity_flip, leds):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None
        set_calls[led_index_list[0] + 2] = None
        set_calls[led_index_list[0] + 3] = None
        set_calls[led_index_list[0] + 4] = None
        set_calls[led_index_list[0] + 5] = None
        set_calls[led_index_list[0] + 6] = None
        set_calls[led_index_list[0] + 7] = None
        set_calls[led_index_list[0] + 8] = None
        set_calls[led_index_list[0] + 9] = None
        set_calls[led_index_list[0] + 10] = None
        set_calls[led_index_list[0] + 11] = None
        set_calls[led_index_list[0] + 12] = None
        set_calls[led_index_list[0] + 13] = None

        return set_calls

class AcctonAS7712_32XLED(cumulus.ledup.TomahawkLEDController):
    led_functions = (AcctonAS7712QSFP28._set_led_common)
    num_leds = 224
    has_phys_id = True

class AcctonAS7712_32XSwitch(cumulus.platform.BCMSwitch):
    ledup = AcctonAS7712_32XLED
    led_controllers = set([0, 1, 2])    
    _qsfp28 = AcctonAS7712QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp28(label="1",  serdes=(48, 49, 50, 51),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(1, 0),),
        _qsfp28(label="2",  serdes=(52, 53, 54, 55),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(1, 14),),
        _qsfp28(label="3",  serdes=(56, 57, 58, 59),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(1, 28),),
        _qsfp28(label="4",  serdes=(60, 61, 62, 63),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(1, 42),),
        _qsfp28(label="5",  serdes=(64, 65, 66, 67),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(1, 56),),
        _qsfp28(label="6",  serdes=(68, 69, 70, 71),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(1, 70),),
        _qsfp28(label="7",  serdes=(72, 73, 74, 75),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(1, 84),),
        _qsfp28(label="8",  serdes=(76, 77, 78, 79),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(1, 98),),
        _qsfp28(label="9",  serdes=(32, 33, 34, 35),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(1, 112),),
        _qsfp28(label="10", serdes=(36, 37, 38, 39),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(1, 126),),
        _qsfp28(label="11", serdes=(40, 41, 42, 43),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(1, 140),),
        _qsfp28(label="12", serdes=(44, 45, 46, 47),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(1, 154),),
        _qsfp28(label="13", serdes=(80, 81, 82, 83),
                  tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=0x6,
                  rx_polarity_flip=0xf, leds=(1, 168),),
        _qsfp28(label="14", serdes=(84, 85, 86, 87),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0,
                  rx_polarity_flip=0xf, leds=(1, 182),),
        _qsfp28(label="15", serdes=(88, 89, 90, 91),
                  tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=0xe,
                  rx_polarity_flip=0xf, leds=(1, 196),),
        _qsfp28(label="16", serdes=(92, 93, 94, 95),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x1,
                  rx_polarity_flip=0x0, leds=(1, 210),),
        _qsfp28(label="17", serdes=(96, 97, 98, 99),
                  tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=0x1,
                  rx_polarity_flip=0x0, leds=(0, 0),),
        _qsfp28(label="18", serdes=(100, 101, 102, 103),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0,
                  rx_polarity_flip=0xf, leds=(0, 14),),
        _qsfp28(label="19", serdes=(104, 105, 106, 107),
                  tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=0xe,
                  rx_polarity_flip=0x0, leds=(0, 28),),
        _qsfp28(label="20", serdes=(108, 109, 110, 111),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x2, leds=(0, 42),),
        _qsfp28(label="21", serdes=(16, 17, 18, 19),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0xf,
                  rx_polarity_flip=0x0, leds=(0, 56),),
        _qsfp28(label="22", serdes=(20, 21, 22, 23),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(0, 70),),
        _qsfp28(label="23", serdes=(24, 25, 26, 27),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(0, 84),),
        _qsfp28(label="24", serdes=(28, 29, 30, 31),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(0, 98),),
        _qsfp28(label="25", serdes=(112, 113, 114, 115),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0xf,
                  rx_polarity_flip=0xa, leds=(0, 112),),
        _qsfp28(label="26", serdes=(116, 117, 118, 119),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(0, 126),),
        _qsfp28(label="27", serdes=(120, 121, 122, 123),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(0, 140),),
        _qsfp28(label="28", serdes=(124, 125, 126, 127),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(0, 154),),
        _qsfp28(label="29", serdes=(0, 1, 2, 3),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(0, 168),),
        _qsfp28(label="30", serdes=(4, 5, 6, 7),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(0, 182),),
        _qsfp28(label="31", serdes=(8, 9, 10, 11),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(0, 196),),
        _qsfp28(label="32", serdes=(12, 13, 14, 15),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0,
                  rx_polarity_flip=0x0, leds=(0, 210),),
    )

class AcctonAS7712_32XCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/accton_as7712_32x_cpld.0'
    
class AcctonAS7712_32XTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = 'Temp sensor behind networking asic'
    driver_path = '/sys/bus/i2c/devices/51-0048'
    temp_fan_min  = 41
    temp_fan_max  = 55
    temp_max  = 61
    temp_crit = 67 

class AcctonAS7712_32XTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Temp sensor in front of networking asic'
    driver_path = '/sys/bus/i2c/devices/51-0049'
    temp_fan_min  = 46
    temp_fan_max  = 60
    temp_max  = 66
    temp_crit = 69 

class AcctonAS7712_32XTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Temp sensor front left'
    driver_path = '/sys/bus/i2c/devices/51-004a'
    temp_fan_min  = 32
    temp_fan_max  = 50
    temp_max  = 56
    temp_crit = 59 

class AcctonAS7712_32XTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    driver_path = '/sys/bus/i2c/devices/51-004b'
    description = 'Temp Sensor Near CPU'
    temp_fan_min = 33 
    temp_fan_max = 50
    temp_max     = 57
    temp_crit    = 60

AcctonAS7712_32X_core_path = '/sys/devices/platform/coretemp.0'

class AcctonAS7712_32XCPUTemp(cumulus.sensors.Temp_Unit):
    description = 'Intel CPU die sensor'
    driver_path = AcctonAS7712_32X_core_path
    temp_fan_min = 58  
    temp_fan_max = 65
    temp_max     = 70
    temp_crit    = 98

class AcctonAS7712_32XTemp5(AcctonAS7712_32XCPUTemp):
    name = 'Temp5'
    driver_hwmon = ['temp2']

class AcctonAS7712_32XTemp6(AcctonAS7712_32XCPUTemp):
    name = 'Temp6'
    driver_hwmon = ['temp3']

class AcctonAS7712_32XTemp7(AcctonAS7712_32XCPUTemp):
    name = 'Temp7'
    driver_hwmon = ['temp4']

class AcctonAS7712_32XTemp8(AcctonAS7712_32XCPUTemp):
    name = 'Temp8'
    driver_path = AcctonAS7712_32X_core_path
    driver_hwmon = ['temp5']

AcctonAS7712_32X_PSU1_i2c_path = '/sys/bus/i2c/devices/11-005b'
AcctonAS7712_32X_PSU2_i2c_path = '/sys/bus/i2c/devices/10-0058'

class AcctonAS7712_32XPSUTemp(cumulus.sensors.Temp_Unit):
    temp_crit = 60
    temp_max = 50
    temp_fan_min = 25
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class AcctonAS7712_32XTemp9(AcctonAS7712_32XPSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS7712_32X_PSU1_i2c_path
    driver_hwmon = ['temp1']

class AcctonAS7712_32XTemp10(AcctonAS7712_32XPSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS7712_32X_PSU1_i2c_path
    driver_hwmon = ['temp2']

class AcctonAS7712_32XTemp11(AcctonAS7712_32XPSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS7712_32X_PSU2_i2c_path
    driver_hwmon = ['temp1']

class AcctonAS7712_32XTemp12(AcctonAS7712_32XPSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS7712_32X_PSU2_i2c_path
    driver_hwmon = ['temp2']    

class AcctonAS7712_32X_ASICTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_fan_min = 67
    temp_fan_max = 82
    temp_max     = 87
    temp_crit    = 95

class AcctonAS7712_32XTemp13(AcctonAS7712_32X_ASICTemp):
    name = 'Temp9'
    driver_hwmon = ['temp1']

class AcctonAS7712_32XTemp14(AcctonAS7712_32X_ASICTemp):
    name = 'Temp10'
    driver_hwmon = ['temp2']

class AcctonAS7712_32XTemp15(AcctonAS7712_32X_ASICTemp):
    name = 'Temp11'
    driver_hwmon = ['temp3']

class AcctonAS7712_32XTemp16(AcctonAS7712_32X_ASICTemp):
    name = 'Temp12'
    driver_hwmon = ['temp4']

class AcctonAS7712_32XTemp17(AcctonAS7712_32X_ASICTemp):
    name = 'Temp13'
    driver_hwmon = ['temp5']

class AcctonAS7712_32XTemp18(AcctonAS7712_32X_ASICTemp):
    name = 'Temp14'
    driver_hwmon = ['temp6']

class AcctonAS7712_32XTemp19(AcctonAS7712_32X_ASICTemp):
    name = 'Temp15'
    driver_hwmon = ['temp7']

class AcctonAS7712_32XTemp20(AcctonAS7712_32X_ASICTemp):
    name = 'Temp16'
    driver_hwmon = ['temp8']

class AcctonAS7712_32X_PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = ['/sbin/rmmod', 'cpr4011']
            retrm =  subprocess.call(cmd)
            if retrm == 0:
                cmd = ['/sbin/modprobe', 'cpr4011']
                subprocess.call(cmd)
            self.init_done = True

class AcctonAS7712_32XPSU1(AcctonAS7712_32X_PSU):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class AcctonAS7712_32XPSU2(AcctonAS7712_32X_PSU):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

# 7712 has 6 fan trays, each of which has a front
# and a rear fan.
class AcctonAS7712_32XFan(cumulus.sensors.Fan_Unit):
    hwmon_search = False
    driver_path = AcctonAS7712_32XCPLD.path
    fan_min = 12000
    # Accton to confirm this value.
    fan_max = 21300
    # The minimum value that the
    # pwm register can hold is 4. Mapping 
    # that to 0~255 range by multiplying 
    # by 17.
    minpwm  = 119

class AcctonAS7712_32XFanR(cumulus.sensors.Fan_Unit):
    hwmon_search = False
    driver_path = AcctonAS7712_32XCPLD.path
    fan_min = 10100
    fan_max = 17700
    minpwm  = 119

class AcctonAS7712_32XFan1(AcctonAS7712_32XFan):
    name = 'Fan1'
    description = 'Fan Tray 1, Fan A'
    # driver is fan speed ...
    driver_hwmon = ['fan1']
    # status is "cpld_hwmon"
    cpld_hwmon = ['fan_1']

    # The CPLD driver only exports "pwm1", which controls all the
    # fans.
    pwm_hwmon = ['pwm1']

class AcctonAS7712_32XFan2(AcctonAS7712_32XFanR):
    name = 'Fan2'
    description = 'Fan Tray 1, Fan B'
    driver_hwmon = ['fan2']
    cpld_hwmon = ['fan_1']

class AcctonAS7712_32XFan3(AcctonAS7712_32XFan):
    name = 'Fan3'
    description = 'Fan Tray 2, Fan A'
    driver_hwmon = ['fan3']
    cpld_hwmon = ['fan_2']

class AcctonAS7712_32XFan4(AcctonAS7712_32XFanR):
    name = 'Fan4'
    description = 'Fan Tray 2, Fan B'
    driver_hwmon = ['fan4']
    cpld_hwmon = ['fan_2']

class AcctonAS7712_32XFan5(AcctonAS7712_32XFan):
    name = 'Fan5'
    description = 'Fan Tray 3, Fan A'
    driver_hwmon = ['fan5']
    cpld_hwmon = ['fan_3']

class AcctonAS7712_32XFan6(AcctonAS7712_32XFanR):
    name = 'Fan6'
    description = 'Fan Tray 3, Fan B'
    driver_hwmon = ['fan6']
    cpld_hwmon = ['fan_3']

class AcctonAS7712_32XFan7(AcctonAS7712_32XFan):
    name = 'Fan7'
    description = 'Fan Tray 4, Fan A'
    driver_hwmon = ['fan7']
    cpld_hwmon = ['fan_4']

class AcctonAS7712_32XFan8(AcctonAS7712_32XFanR):
    name = 'Fan8'
    description = 'Fan Tray 4, Fan B'
    driver_hwmon = ['fan8']
    cpld_hwmon = ['fan_4']

class AcctonAS7712_32XFan9(AcctonAS7712_32XFan):
    name = 'Fan9'
    description = 'Fan Tray 5, Fan A'
    driver_hwmon = ['fan9']
    cpld_hwmon = ['fan_5']

class AcctonAS7712_32XFan10(AcctonAS7712_32XFanR):
    name = 'Fan10'
    description = 'Fan Tray 5, Fan B'
    driver_hwmon = ['fan10']
    cpld_hwmon = ['fan_5']

class AcctonAS7712_32XFan11(AcctonAS7712_32XFan):
    name = 'Fan11'
    description = 'Fan Tray 6, Fan A'
    driver_hwmon = ['fan11']
    cpld_hwmon = ['fan_6']

class AcctonAS7712_32XFan12(AcctonAS7712_32XFanR):
    name = 'Fan12'
    description = 'Fan Tray 6, Fan B'
    driver_hwmon = ['fan12']
    cpld_hwmon = ['fan_6']


class AcctonAS7712_32XSensors():
    units = ( AcctonAS7712_32XTemp1(),
              AcctonAS7712_32XTemp2(),
              AcctonAS7712_32XTemp3(),
              AcctonAS7712_32XTemp4(),
              AcctonAS7712_32XTemp5(),
              AcctonAS7712_32XTemp6(),
              AcctonAS7712_32XTemp7(),
              AcctonAS7712_32XTemp8(),
              AcctonAS7712_32XTemp9(),
              AcctonAS7712_32XTemp10(),
              AcctonAS7712_32XTemp11(),
              AcctonAS7712_32XTemp12(),
              AcctonAS7712_32XTemp13(),
              AcctonAS7712_32XTemp14(),
              AcctonAS7712_32XTemp15(),
              AcctonAS7712_32XTemp16(),
              AcctonAS7712_32XTemp17(),
              AcctonAS7712_32XTemp18(),
              AcctonAS7712_32XTemp19(),
              AcctonAS7712_32XTemp20(),
              AcctonAS7712_32XPSU1(),
              AcctonAS7712_32XPSU2(),
              AcctonAS7712_32XFan1(),
              AcctonAS7712_32XFan2(),
              AcctonAS7712_32XFan3(),
              AcctonAS7712_32XFan4(),
              AcctonAS7712_32XFan5(),
              AcctonAS7712_32XFan6(),
              AcctonAS7712_32XFan7(),
              AcctonAS7712_32XFan8(),
              AcctonAS7712_32XFan9(),
              AcctonAS7712_32XFan10(),
              AcctonAS7712_32XFan11(),
              AcctonAS7712_32XFan12(),
   )

class AcctonAS7712_32XPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (AcctonAS7712_32XFan1(),
            AcctonAS7712_32XFan2(),
            AcctonAS7712_32XFan3(),
            AcctonAS7712_32XFan4(),
            AcctonAS7712_32XFan5(),
            AcctonAS7712_32XFan6(),
            AcctonAS7712_32XFan7(),
            AcctonAS7712_32XFan8(),
            AcctonAS7712_32XFan9(),
            AcctonAS7712_32XFan10(),
            AcctonAS7712_32XFan11(),
            AcctonAS7712_32XFan12(),
            )
    temps = (AcctonAS7712_32XTemp1(),
             AcctonAS7712_32XTemp2(),
             AcctonAS7712_32XTemp3(),
             AcctonAS7712_32XTemp4(),
             AcctonAS7712_32XTemp5(),
             AcctonAS7712_32XTemp6(),
             AcctonAS7712_32XTemp7(),
             AcctonAS7712_32XTemp8(),
             AcctonAS7712_32XTemp9(),
             AcctonAS7712_32XTemp10(),
             AcctonAS7712_32XTemp11(),
             AcctonAS7712_32XTemp12(),
             AcctonAS7712_32XTemp13(),
             AcctonAS7712_32XTemp14(),
             AcctonAS7712_32XTemp15(),
             AcctonAS7712_32XTemp16(),
             AcctonAS7712_32XTemp17(),
             AcctonAS7712_32XTemp18(),
             AcctonAS7712_32XTemp19(),
             AcctonAS7712_32XTemp20(),
             )

class AcctonAS7712_32XPWMs():
    units = (AcctonAS7712_32XPWM1(),
             )

class AcctonAS7712_32X(cumulus.platform.Platform):
    name = 'accton,as7712_32x'
    chip = cumulus.platform.TomahawkChip()
    switch = AcctonAS7712_32XSwitch(chip)
    cpld = AcctonAS7712_32XCPLD()
    statusleds = None
    pwms = AcctonAS7712_32XPWMs()
    sensors = AcctonAS7712_32XSensors()

Platform = AcctonAS7712_32X
