#
# Copyright (c) 2018 Cumulus Networks, Inc.  All Rights Reserved.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
from json import loads
import syslog

class Act7816QSFP28(cumulus.platform.TomahawkTwo100GPort):
    '''
        Each port has 14 bits of LED data controlling four RGB LEDs.  The
        first two bits defines the port lane number.  Then each lane
        uses three bits to light its LEDs.
    '''
    _set_led_common = '''
;;;
;;; common LED routines
;;;
set_00:
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ZERO
        pack
        ret

set_led_green:
	pushst  PORTSTATUS_ZERO
	pack
	pushst  PORTSTATUS_ZERO
	pack

check_and_pack:
        ld      b, (ID_ENABLE_P)
        tst     b, ID_ENABLE_BIT
        jc      id_mode

        port  a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                       ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
	pack
	ret

id_mode:
        push    0
        cmp     a, (ID_PORT_P)
        jnz     id_pack
        ld      b, (WIGGLE_STATE_P)
        and     b, (WAG_STATE_P)
        push    b
id_pack:
        pack
        ret

set_led_off:
	pushst  PORTSTATUS_ZERO
	pack
	pushst  PORTSTATUS_ZERO
	pack
	pushst  PORTSTATUS_ZERO
	pack
	ret

'''
    _set_led_1x = '''
;;;
;;; In 1x mode, only the first LED is used to indicate link/activity. The
;;; other three LEDs are off.
;;;
set_led:
        call    set_00
        call    set_led_green
        call    set_led_off
        call    set_led_off
        call    set_led_off
        ret
'''

    _set_led_2x = '''
;;;
;;; In 2x mode, the first and third LEDs are used the indicate link/activity.
;;; The second and fourth LEDs are off.
;;;
set_led:
        call    set_00
        call    set_led_green
        call    set_led_off
        inc     a
        inc     a
        call    set_led_green
        call    set_led_off
        ret
'''

    _set_led_4x = '''
;;;
;;; In 4x mode, all four LEDs are used to indicate link/activity.
;;;
set_led:
        call    set_00
        call    set_led_green
        inc     a
        call    set_led_green
        inc     a
        call    set_led_green
        inc     a
        call    set_led_green
        ret
'''

    def __init__(self, label, adj_label, serdes, leds, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None, blocked_adj_split=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TomahawkTwo100GPort.__init__(self,
                                                      label=label,
                                                      adj_label=adj_label,
                                                      serdes=serdes,
                                                      leds=leds,
                                                      phy_types=None,
                                                      phy_addrs=None,
                                                      phy_layer=sfp.type.upper(),
                                                      tx_lane_map=tx_lane_map,
                                                      rx_lane_map=rx_lane_map,
                                                      tx_polarity_flip=tx_polarity_flip,
                                                      rx_polarity_flip=rx_polarity_flip,
                                                      pre_emphasis=pre_emphasis,
                                                      blocked_adj_split=blocked_adj_split,
                                                      unit=0,
                                                      oversub=True)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None
        set_calls[led_index_list[0]+8] = None
        set_calls[led_index_list[0]+9] = None
        set_calls[led_index_list[0]+10] = None
        set_calls[led_index_list[0]+11] = None
        set_calls[led_index_list[0]+12] = None
        set_calls[led_index_list[0]+13] = None

        return set_calls

# Switch Ports

class AcctonAS7816_64XLED(cumulus.ledup.TomahawkLEDController):
    led_functions = (Act7816QSFP28._set_led_common)
    num_leds = 224
    has_phys_id = True
    default_pad = 0

class AcctonAS7816_64XSwitch(cumulus.platform.BCMSwitch):
    ledup = AcctonAS7816_64XLED
    led_controllers = set([0, 1, 2, 3])
    _qsfp28 = Act7816QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp28(label="1",  serdes=( 72,  73,  74,  75), leds=(1,  28), tx_lane_map=0x3021, rx_lane_map=0x3210, tx_polarity_flip=0x0004, rx_polarity_flip=0x0008, adj_label=None),
        _qsfp28(label="2",  serdes=( 64,  65,  66,  67), leds=(1,   0), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0001, rx_polarity_flip=0x0000, adj_label="1", blocked_adj_split=4),
        _qsfp28(label="3",  serdes=( 80,  81,  82,  83), leds=(1,  56), tx_lane_map=0x1032, rx_lane_map=0x1230, tx_polarity_flip=0x0007, rx_polarity_flip=0x0005, adj_label=None),
        _qsfp28(label="4",  serdes=( 88,  89,  90,  91), leds=(1,  84), tx_lane_map=0x1203, rx_lane_map=0x0132, tx_polarity_flip=0x000d, rx_polarity_flip=0x0009, adj_label="3", blocked_adj_split=4),

        _qsfp28(label="5",  serdes=(104, 105, 106, 107), leds=(1, 140), tx_lane_map=0x0231, rx_lane_map=0x3210, tx_polarity_flip=0x000a, rx_polarity_flip=0x0008, adj_label=None),
        _qsfp28(label="6",  serdes=( 96,  97,  98,  99), leds=(1, 112), tx_lane_map=0x3210, rx_lane_map=0x0213, tx_polarity_flip=0x0003, rx_polarity_flip=0x0009, adj_label="5", blocked_adj_split=4),
        _qsfp28(label="7",  serdes=(112, 113, 114, 115), leds=(1, 168), tx_lane_map=0x0312, rx_lane_map=0x3021, tx_polarity_flip=0x0009, rx_polarity_flip=0x0006, adj_label=None),
        _qsfp28(label="8",  serdes=(120, 121, 122, 123), leds=(1, 196), tx_lane_map=0x2130, rx_lane_map=0x3021, tx_polarity_flip=0x0007, rx_polarity_flip=0x0006, adj_label="7", blocked_adj_split=4),

        _qsfp28(label="9",  serdes=( 40,  41,  42,  43), leds=(0, 140), tx_lane_map=0x1302, rx_lane_map=0x0132, tx_polarity_flip=0x0001, rx_polarity_flip=0x0000, adj_label=None),
        _qsfp28(label="10", serdes=( 32,  33,  34,  35), leds=(0, 112), tx_lane_map=0x0213, rx_lane_map=0x2310, tx_polarity_flip=0x0003, rx_polarity_flip=0x0002, adj_label="9", blocked_adj_split=4),
        _qsfp28(label="11", serdes=( 48,  49,  50,  51), leds=(0, 168), tx_lane_map=0x3102, rx_lane_map=0x3210, tx_polarity_flip=0x000b, rx_polarity_flip=0x0005, adj_label=None),
        _qsfp28(label="12", serdes=( 56,  57,  58,  59), leds=(0, 196), tx_lane_map=0x1302, rx_lane_map=0x3210, tx_polarity_flip=0x0008, rx_polarity_flip=0x0005, adj_label="11", blocked_adj_split=4),

        _qsfp28(label="13", serdes=(136, 137, 138, 139), leds=(2,  28), tx_lane_map=0x0213, rx_lane_map=0x3210, tx_polarity_flip=0x0001, rx_polarity_flip=0x0004, adj_label=None),
        _qsfp28(label="14", serdes=(128, 129, 130, 131), leds=(2,   0), tx_lane_map=0x1203, rx_lane_map=0x3021, tx_polarity_flip=0x000f, rx_polarity_flip=0x0001, adj_label="13", blocked_adj_split=4),
        _qsfp28(label="15", serdes=(144, 145, 146, 147), leds=(2,  56), tx_lane_map=0x2301, rx_lane_map=0x0213, tx_polarity_flip=0x0008, rx_polarity_flip=0x0003, adj_label=None),
        _qsfp28(label="16", serdes=(152, 153, 154, 155), leds=(2,  84), tx_lane_map=0x1302, rx_lane_map=0x0213, tx_polarity_flip=0x0002, rx_polarity_flip=0x000d, adj_label="15", blocked_adj_split=4),

        _qsfp28(label="17", serdes=(172, 173, 174, 175), leds=(2, 154), tx_lane_map=0x1203, rx_lane_map=0x1032, tx_polarity_flip=0x000b, rx_polarity_flip=0x0000, adj_label=None),
        _qsfp28(label="18", serdes=(164, 165, 166, 167), leds=(2, 126), tx_lane_map=0x2130, rx_lane_map=0x1230, tx_polarity_flip=0x0005, rx_polarity_flip=0x000d, adj_label="17", blocked_adj_split=4),
        _qsfp28(label="19", serdes=(180, 181, 182, 183), leds=(2, 182), tx_lane_map=0x3120, rx_lane_map=0x0312, tx_polarity_flip=0x0004, rx_polarity_flip=0x0006, adj_label=None),
        _qsfp28(label="20", serdes=(188, 189, 190, 191), leds=(2, 210), tx_lane_map=0x3210, rx_lane_map=0x0132, tx_polarity_flip=0x0001, rx_polarity_flip=0x000e, adj_label="19", blocked_adj_split=4),

        _qsfp28(label="21", serdes=( 12,  13,  14,  15), leds=(0,  42), tx_lane_map=0x3210, rx_lane_map=0x3120, tx_polarity_flip=0x0008, rx_polarity_flip=0x000d, adj_label=None),
        _qsfp28(label="22", serdes=(  4,   5,   6,   7), leds=(0,  14), tx_lane_map=0x0321, rx_lane_map=0x0213, tx_polarity_flip=0x000b, rx_polarity_flip=0x0004, adj_label="21", blocked_adj_split=4),
        _qsfp28(label="23", serdes=( 28,  29,  30,  31), leds=(0,  98), tx_lane_map=0x2130, rx_lane_map=0x3021, tx_polarity_flip=0x0001, rx_polarity_flip=0x0009, adj_label=None),
        _qsfp28(label="24", serdes=( 20,  21,  22,  23), leds=(0,  70), tx_lane_map=0x0123, rx_lane_map=0x0321, tx_polarity_flip=0x000e, rx_polarity_flip=0x0005, adj_label="23", blocked_adj_split=4),

        _qsfp28(label="25", serdes=(204, 205, 206, 207), leds=(3,  42), tx_lane_map=0x1230, rx_lane_map=0x0132, tx_polarity_flip=0x000a, rx_polarity_flip=0x0007, adj_label=None),
        _qsfp28(label="26", serdes=(196, 197, 198, 199), leds=(3,  14), tx_lane_map=0x3021, rx_lane_map=0x1230, tx_polarity_flip=0x0005, rx_polarity_flip=0x0000, adj_label="25", blocked_adj_split=4),
        _qsfp28(label="27", serdes=(212, 213, 214, 215), leds=(3,  70), tx_lane_map=0x1230, rx_lane_map=0x3210, tx_polarity_flip=0x0008, rx_polarity_flip=0x000a, adj_label=None),
        _qsfp28(label="28", serdes=(220, 221, 222, 223), leds=(3,  98), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0000, rx_polarity_flip=0x0003, adj_label="27", blocked_adj_split=4),

        _qsfp28(label="29", serdes=(228, 229, 230, 231), leds=(3, 126), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=0x0002, rx_polarity_flip=0x000b, adj_label=None),
        _qsfp28(label="30", serdes=(236, 237, 238, 239), leds=(3, 154), tx_lane_map=0x3210, rx_lane_map=0x0123, tx_polarity_flip=0x0003, rx_polarity_flip=0x000d, adj_label="29", blocked_adj_split=4),
        _qsfp28(label="31", serdes=(244, 245, 246, 247), leds=(3, 182), tx_lane_map=0x3210, rx_lane_map=0x0213, tx_polarity_flip=0x0005, rx_polarity_flip=0x000c, adj_label=None),
        _qsfp28(label="32", serdes=(252, 253, 254, 255), leds=(3, 210), tx_lane_map=0x0312, rx_lane_map=0x0213, tx_polarity_flip=0x0002, rx_polarity_flip=0x0008, adj_label="31", blocked_adj_split=4),

        _qsfp28(label="33", serdes=( 68,  69,  70,  71), leds=(1,  14), tx_lane_map=0x3102, rx_lane_map=0x1032, tx_polarity_flip=0x000c, rx_polarity_flip=0x0009, adj_label=None),
        _qsfp28(label="34", serdes=( 76,  77,  78,  79), leds=(1,  42), tx_lane_map=0x3021, rx_lane_map=0x1230, tx_polarity_flip=0x0004, rx_polarity_flip=0x0005, adj_label="33", blocked_adj_split=4),
        _qsfp28(label="35", serdes=( 92,  93,  94,  95), leds=(1,  98), tx_lane_map=0x0231, rx_lane_map=0x1032, tx_polarity_flip=0x000e, rx_polarity_flip=0x000f, adj_label=None),
        _qsfp28(label="36", serdes=( 84,  85,  86,  87), leds=(1,  70), tx_lane_map=0x1230, rx_lane_map=0x0312, tx_polarity_flip=0x0005, rx_polarity_flip=0x0008, adj_label="35", blocked_adj_split=4),

        _qsfp28(label="37", serdes=(100, 101, 102, 103), leds=(1, 126), tx_lane_map=0x0213, rx_lane_map=0x1302, tx_polarity_flip=0x000a, rx_polarity_flip=0x0001, adj_label=None),
        _qsfp28(label="38", serdes=(108, 109, 110, 111), leds=(1, 154), tx_lane_map=0x1032, rx_lane_map=0x0213, tx_polarity_flip=0x000e, rx_polarity_flip=0x0003, adj_label="37", blocked_adj_split=4),
        _qsfp28(label="39", serdes=(124, 125, 126, 127), leds=(1, 210), tx_lane_map=0x1203, rx_lane_map=0x2130, tx_polarity_flip=0x000e, rx_polarity_flip=0x000c, adj_label=None),
        _qsfp28(label="40", serdes=(116, 117, 118, 119), leds=(1, 182), tx_lane_map=0x2310, rx_lane_map=0x3102, tx_polarity_flip=0x0005, rx_polarity_flip=0x0005, adj_label="39", blocked_adj_split=4),

        _qsfp28(label="41", serdes=( 36,  37,  38,  39), leds=(0, 126), tx_lane_map=0x1302, rx_lane_map=0x3210, tx_polarity_flip=0x000d, rx_polarity_flip=0x0001, adj_label=None),
        _qsfp28(label="42", serdes=( 44,  45,  46,  47), leds=(0, 154), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x000a, rx_polarity_flip=0x0000, adj_label="41", blocked_adj_split=4),
        _qsfp28(label="43", serdes=( 60,  61,  62,  63), leds=(0, 210), tx_lane_map=0x1230, rx_lane_map=0x3210, tx_polarity_flip=0x0006, rx_polarity_flip=0x0002, adj_label=None),
        _qsfp28(label="44", serdes=( 52,  53,  54,  55), leds=(0, 182), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0007, rx_polarity_flip=0x0004, adj_label="43", blocked_adj_split=4),

        _qsfp28(label="45", serdes=(132, 133, 134, 135), leds=(2,  14), tx_lane_map=0x2310, rx_lane_map=0x0312, tx_polarity_flip=0x000a, rx_polarity_flip=0x0004, adj_label=None),
        _qsfp28(label="46", serdes=(140, 141, 142, 143), leds=(2,  42), tx_lane_map=0x0123, rx_lane_map=0x3012, tx_polarity_flip=0x0003, rx_polarity_flip=0x0005, adj_label="45", blocked_adj_split=4),
        _qsfp28(label="47", serdes=(156, 157, 158, 159), leds=(2,  98), tx_lane_map=0x0231, rx_lane_map=0x2103, tx_polarity_flip=0x0002, rx_polarity_flip=0x0004, adj_label=None),
        _qsfp28(label="48", serdes=(148, 149, 150, 151), leds=(2,  70), tx_lane_map=0x1023, rx_lane_map=0x3120, tx_polarity_flip=0x0009, rx_polarity_flip=0x0002, adj_label="47", blocked_adj_split=4),

        _qsfp28(label="49", serdes=(160, 161, 162, 163), leds=(2, 112), tx_lane_map=0x1023, rx_lane_map=0x3012, tx_polarity_flip=0x0002, rx_polarity_flip=0x0003, adj_label=None),
        _qsfp28(label="50", serdes=(168, 169, 170, 171), leds=(2, 140), tx_lane_map=0x2031, rx_lane_map=0x3210, tx_polarity_flip=0x0001, rx_polarity_flip=0x0002, adj_label="49", blocked_adj_split=4),
        _qsfp28(label="51", serdes=(184, 185, 186, 187), leds=(2, 196), tx_lane_map=0x3021, rx_lane_map=0x2310, tx_polarity_flip=0x0002, rx_polarity_flip=0x0005, adj_label=None),
        _qsfp28(label="52", serdes=(176, 177, 178, 179), leds=(2, 168), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=0x0007, rx_polarity_flip=0x000b, adj_label="51", blocked_adj_split=4),

        _qsfp28(label="53", serdes=(  0,   1,   2,   3), leds=(0,   0), tx_lane_map=0x1203, rx_lane_map=0x2130, tx_polarity_flip=0x0008, rx_polarity_flip=0x000d, adj_label=None),
        _qsfp28(label="54", serdes=(  8,   9,  10,  11), leds=(0,  28), tx_lane_map=0x1230, rx_lane_map=0x1203, tx_polarity_flip=0x0005, rx_polarity_flip=0x0002, adj_label="53", blocked_adj_split=4),
        _qsfp28(label="55", serdes=( 24,  25,  26,  27), leds=(0,  84), tx_lane_map=0x0132, rx_lane_map=0x3102, tx_polarity_flip=0x000a, rx_polarity_flip=0x0004, adj_label=None),
        _qsfp28(label="56", serdes=( 16,  17,  18,  19), leds=(0,  56), tx_lane_map=0x3021, rx_lane_map=0x3120, tx_polarity_flip=0x0004, rx_polarity_flip=0x0002, adj_label="55", blocked_adj_split=4),

        _qsfp28(label="57", serdes=(192, 193, 194, 195), leds=(3,   0), tx_lane_map=0x1230, rx_lane_map=0x2103, tx_polarity_flip=0x0004, rx_polarity_flip=0x000a, adj_label=None),
        _qsfp28(label="58", serdes=(200, 201, 202, 203), leds=(3,  28), tx_lane_map=0x3021, rx_lane_map=0x0321, tx_polarity_flip=0x000b, rx_polarity_flip=0x0004, adj_label="57", blocked_adj_split=4),
        _qsfp28(label="59", serdes=(216, 217, 218, 219), leds=(3,  84), tx_lane_map=0x2031, rx_lane_map=0x1203, tx_polarity_flip=0x0005, rx_polarity_flip=0x000d, adj_label=None),
        _qsfp28(label="60", serdes=(208, 209, 210, 211), leds=(3,  56), tx_lane_map=0x3102, rx_lane_map=0x0321, tx_polarity_flip=0x0004, rx_polarity_flip=0x0004, adj_label="59", blocked_adj_split=4),

        _qsfp28(label="61", serdes=(224, 225, 226, 227), leds=(3, 112), tx_lane_map=0x2031, rx_lane_map=0x2103, tx_polarity_flip=0x000d, rx_polarity_flip=0x000c, adj_label=None),
        _qsfp28(label="62", serdes=(232, 233, 234, 235), leds=(3, 140), tx_lane_map=0x0312, rx_lane_map=0x2130, tx_polarity_flip=0x0004, rx_polarity_flip=0x0006, adj_label="61", blocked_adj_split=4),
        _qsfp28(label="63", serdes=(248, 249, 250, 251), leds=(3, 196), tx_lane_map=0x1302, rx_lane_map=0x1302, tx_polarity_flip=0x000f, rx_polarity_flip=0x000a, adj_label=None),
        _qsfp28(label="64", serdes=(240, 241, 242, 243), leds=(3, 168), tx_lane_map=0x2301, rx_lane_map=0x3012, tx_polarity_flip=0x0005, rx_polarity_flip=0x000b, adj_label="63", blocked_adj_split=4),
    )

class AcctonAS7816_64XCPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/platform/accton_as7816_64x_cpld.0'

# Intel CPU temp sensors

class AcctonAS7816_64XCoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    no_shutdown = True

class AcctonAS7816_64XCPUTemp1(AcctonAS7816_64XCoreTemp):
    name = 'Temp1'
    description = 'Physical id 0'
    driver_hwmon = ['temp1']

class AcctonAS7816_64XCPUTemp2(AcctonAS7816_64XCoreTemp):
    name = 'Temp2'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class AcctonAS7816_64XCPUTemp3(AcctonAS7816_64XCoreTemp):
    name = 'Temp3'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class AcctonAS7816_64XCPUTemp4(AcctonAS7816_64XCoreTemp):
    name = 'Temp4'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class AcctonAS7816_64XCPUTemp5(AcctonAS7816_64XCoreTemp):
    name = 'Temp5'
    description = 'Core 3'
    driver_hwmon = ['temp5']

# Tomahawk2 internal temp sensors

class AcctonAS7816_64XTomahawkTwoTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 105
    temp_crit    = 110

class AcctonAS7816_64XTomahawkTwoTemp1(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp6'
    driver_hwmon = ['temp1']

class AcctonAS7816_64XTomahawkTwoTemp2(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp7'
    driver_hwmon = ['temp2']

class AcctonAS7816_64XTomahawkTwoTemp3(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp8'
    driver_hwmon = ['temp3']

class AcctonAS7816_64XTomahawkTwoTemp4(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp9'
    driver_hwmon = ['temp4']

class AcctonAS7816_64XTomahawkTwoTemp5(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp10'
    driver_hwmon = ['temp5']

class AcctonAS7816_64XTomahawkTwoTemp6(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp11'
    driver_hwmon = ['temp6']

class AcctonAS7816_64XTomahawkTwoTemp7(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp12'
    driver_hwmon = ['temp7']

class AcctonAS7816_64XTomahawkTwoTemp8(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp13'
    driver_hwmon = ['temp8']

# Switch board temp sensors (front to back)

class AcctonAS7816_64XTemp1F2B(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp14'
    description = 'Switch board sensor'
    driver_path = '/sys/bus/i2c/devices/19-0048'
    temp_fan_min = 57
    temp_fan_max = 67

class AcctonAS7816_64XTemp2F2B(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp15'
    description = 'Switch board sensor'
    driver_path = '/sys/bus/i2c/devices/19-0049'
    temp_fan_min = 60
    temp_fan_max = 70

class AcctonAS7816_64XTemp3F2B(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp16'
    description = 'Switch board sensor'
    driver_path = '/sys/bus/i2c/devices/19-004a'
    temp_fan_min = 40
    temp_fan_max = 55

class AcctonAS7816_64XTemp4F2B(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp17'
    description = 'Switch board sensor'
    driver_path = '/sys/bus/i2c/devices/19-004b'
    temp_fan_min = 39
    temp_fan_max = 54

# Switch board temp sensors (back to front)

class AcctonAS7816_64XTemp1B2F(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp14'
    description = 'Switch board sensor'
    driver_path = '/sys/bus/i2c/devices/19-0048'
    temp_fan_min = 52
    temp_fan_max = 66

class AcctonAS7816_64XTemp2B2F(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp15'
    description = 'Switch board sensor'
    driver_path = '/sys/bus/i2c/devices/19-0049'
    temp_fan_min = 41
    temp_fan_max = 57

class AcctonAS7816_64XTemp3B2F(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp16'
    description = 'Switch board sensor'
    driver_path = '/sys/bus/i2c/devices/19-004a'
    temp_fan_min = 34
    temp_fan_max = 51

class AcctonAS7816_64XTemp4B2F(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp17'
    description = 'Switch board sensor'
    driver_path = '/sys/bus/i2c/devices/19-004b'
    temp_fan_min = 27
    temp_fan_max = 47

# Fan board temp sensors (front to back)

class AcctonAS7816_64XTemp5F2B(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp18'
    description = 'Fan board sensor'
    driver_path = '/sys/bus/i2c/devices/18-004d'
    temp_fan_min = 36
    temp_fan_max = 53

class AcctonAS7816_64XTemp6F2B(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp19'
    description = 'Fan board sensor'
    driver_path = '/sys/bus/i2c/devices/18-004e'
    temp_fan_min = 38
    temp_fan_max = 54

# Fan board temp sensors (back to front)

class AcctonAS7816_64XTemp6B2F(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp19'
    description = 'Fan board sensor'
    driver_path = '/sys/bus/i2c/devices/18-004e'
    temp_fan_min = 26
    temp_fan_max = 47

class AcctonAS7816_64XTemp5B2F(AcctonAS7816_64XTomahawkTwoTemp):
    name = 'Temp18'
    description = 'Fan board sensor'
    driver_path = '/sys/bus/i2c/devices/18-004d'
    temp_fan_min = 26
    temp_fan_max = 47

# Fans

class AcctonAS7816_64XFanBase(cumulus.sensors.Fan_Unit):
    cpld_path = AcctonAS7816_64XCPLD.path
    driver_path = cpld_path
    pwm_hwmon = ['fan_pwm']
    hwmon_search = False
    minpwm = 7
    maxpwm = 15

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.fan_present) == '1':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class AcctonAS7816_64XFanFront(AcctonAS7816_64XFanBase):
    fan_min = 0
    fan_max = 18200

class AcctonAS7816_64XFanRear(AcctonAS7816_64XFanBase):
    fan_min = 0
    fan_max = 14700

class AcctonAS7816_64XFan1(AcctonAS7816_64XFanFront):
    fan_present = 'fantray1_present'
    name = 'Fan1'
    description = 'Fan Tray 1 Front'
    driver_hwmon = ['fan1']
    cpld_hwmon = ['fantray1']

class AcctonAS7816_64XFan2(AcctonAS7816_64XFanRear):
    fan_present = 'fantray1_present'
    name = 'Fan2'
    description = 'Fan Tray 1 Rear'
    driver_hwmon = ['fan5']
    cpld_hwmon = ['fantray1']

class AcctonAS7816_64XFan3(AcctonAS7816_64XFanFront):
    fan_present = 'fantray2_present'
    name = 'Fan3'
    description = 'Fan Tray 2 Front'
    driver_hwmon = ['fan2']
    cpld_hwmon = ['fantray2']

class AcctonAS7816_64XFan4(AcctonAS7816_64XFanRear):
    fan_present = 'fantray2_present'
    name = 'Fan4'
    description = 'Fan Tray 2 Rear'
    driver_hwmon = ['fan6']
    cpld_hwmon = ['fantray2']

class AcctonAS7816_64XFan5(AcctonAS7816_64XFanFront):
    fan_present = 'fantray3_present'
    name = 'Fan5'
    description = 'Fan Tray 3 Front'
    driver_hwmon = ['fan3']
    cpld_hwmon = ['fantray3']

class AcctonAS7816_64XFan6(AcctonAS7816_64XFanRear):
    fan_present = 'fantray3_present'
    name = 'Fan6'
    description = 'Fan Tray 3 Rear'
    driver_hwmon = ['fan7']
    cpld_hwmon = ['fantray3']

class AcctonAS7816_64XFan7(AcctonAS7816_64XFanFront):
    fan_present = 'fantray4_present'
    name = 'Fan7'
    description = 'Fan Tray 4 Front'
    driver_hwmon = ['fan4']
    cpld_hwmon = ['fantray4']

class AcctonAS7816_64XFan8(AcctonAS7816_64XFanRear):
    fan_present = 'fantray4_present'
    name = 'Fan8'
    description = 'Fan Tray 4 Rear'
    driver_hwmon = ['fan8']
    cpld_hwmon = ['fantray4']

# PSUs

class AcctonAS7816_64XPSU(cumulus.sensors.PSU_Unit):
    cpld_path = AcctonAS7816_64XCPLD.path

class AcctonAS7816_64XPSU1(AcctonAS7816_64XPSU):
    name = 'PSU1'
    description = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class AcctonAS7816_64XPSU2(AcctonAS7816_64XPSU):
    name = 'PSU2'
    description = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

# LEDs

'''
   Accton AS7816-64X LEDs

   System:
       green   - all systems OK
       amber   - service required
   Locator:
       off     - not used
   Fan:
       green   - fans OK
       amber   - fan failure
   2 x PSU:
       green   - PSU OK
       amber   - PSU failure
'''
class AcctonAS7816_64XLED(cumulus.sensors.StatusLED_Unit):
    led_path = AcctonAS7816_64XCPLD.path

class AcctonAS7816_64XSystemLED(AcctonAS7816_64XLED):
    name = 'Diag LED'
    led_location = 'led_diag'
    good_led_color = 'green'
    bad_led_color = 'red'

class AcctonAS7816_64XFanLED(AcctonAS7816_64XLED):
    name = 'Fan LED'
    led_location = 'led_fan'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6', 'Fan7', 'Fan8']
    good_led_color = 'green'
    bad_led_color = 'red'

class AcctonAS7816_64XPSU1LED(AcctonAS7816_64XLED):
    name = 'PSU1 LED'
    led_location = 'led_psu1'
    sources = ['PSU1']
    good_led_color = 'green'
    bad_led_color = 'red'

class AcctonAS7816_64XPSU2LED(AcctonAS7816_64XLED):
    name = 'PSU2 LED'
    led_location = 'led_psu2'
    sources = ['PSU2']
    good_led_color = 'green'
    bad_led_color = 'red'

# airflow direction

def AcctonAS7816_64X_airflow():
    try:
        product = loads(subprocess.check_output(['/usr/cumulus/bin/decode-syseeprom', '-j']))['tlv']['Product Name']['value']
    except Exception:
        syslog.syslog(syslog.LOG_WARNING, "Unable to read product name.  Defaulting to back-to-front airflow settings.")
        return 'B2F'
    if product.startswith('7816-64X'):
        if product.find('F') >= 0:
            return 'F2B'
        elif product.find('B') >= 0:
            return 'B2F'
        else:
            syslog.syslog(syslog.LOG_WARNING, "Unrecognized product name.  Defaulting to back-to-front airflow settings.")
            return 'B2F'
    else:
        syslog.syslog(syslog.LOG_WARNING, "Unknown product name.  Defaulting to back-to-front airflow settings.")
        return 'B2F'

# Units

class AcctonAS7816_64XSensors():
    units = (AcctonAS7816_64XCPUTemp1(),
             AcctonAS7816_64XCPUTemp2(),
             AcctonAS7816_64XCPUTemp3(),
             AcctonAS7816_64XCPUTemp4(),
             AcctonAS7816_64XCPUTemp5(),
             AcctonAS7816_64XTomahawkTwoTemp1(),
             AcctonAS7816_64XTomahawkTwoTemp2(),
             AcctonAS7816_64XTomahawkTwoTemp3(),
             AcctonAS7816_64XTomahawkTwoTemp4(),
             AcctonAS7816_64XTomahawkTwoTemp5(),
             AcctonAS7816_64XTomahawkTwoTemp6(),
             AcctonAS7816_64XTomahawkTwoTemp7(),
             AcctonAS7816_64XTomahawkTwoTemp8(),
             AcctonAS7816_64XFan1(),
             AcctonAS7816_64XFan2(),
             AcctonAS7816_64XFan3(),
             AcctonAS7816_64XFan4(),
             AcctonAS7816_64XFan5(),
             AcctonAS7816_64XFan6(),
             AcctonAS7816_64XFan7(),
             AcctonAS7816_64XFan8(),
             AcctonAS7816_64XPSU1(),
             AcctonAS7816_64XPSU2(),
             )
    f2b_temps = (AcctonAS7816_64XTemp1F2B(),
                 AcctonAS7816_64XTemp2F2B(),
                 AcctonAS7816_64XTemp3F2B(),
                 AcctonAS7816_64XTemp4F2B(),
                 AcctonAS7816_64XTemp5F2B(),
                 AcctonAS7816_64XTemp6F2B(),
                 )
    b2f_temps = (AcctonAS7816_64XTemp1B2F(),
                 AcctonAS7816_64XTemp2B2F(),
                 AcctonAS7816_64XTemp3B2F(),
                 AcctonAS7816_64XTemp4B2F(),
                 AcctonAS7816_64XTemp5B2F(),
                 AcctonAS7816_64XTemp6B2F(),
                 )
    if AcctonAS7816_64X_airflow() == 'B2F':
        units = units + b2f_temps
    else:
        units = units + f2b_temps

class AcctonAS7816_64XPWM(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (AcctonAS7816_64XFan1(),
            AcctonAS7816_64XFan2(),
            AcctonAS7816_64XFan3(),
            AcctonAS7816_64XFan4(),
            AcctonAS7816_64XFan5(),
            AcctonAS7816_64XFan6(),
            AcctonAS7816_64XFan7(),
            AcctonAS7816_64XFan8(),
            )
    temps = (AcctonAS7816_64XCPUTemp1(),
             AcctonAS7816_64XCPUTemp2(),
             AcctonAS7816_64XCPUTemp3(),
             AcctonAS7816_64XCPUTemp4(),
             AcctonAS7816_64XCPUTemp5(),
             AcctonAS7816_64XTomahawkTwoTemp1(),
             AcctonAS7816_64XTomahawkTwoTemp2(),
             AcctonAS7816_64XTomahawkTwoTemp3(),
             AcctonAS7816_64XTomahawkTwoTemp4(),
             AcctonAS7816_64XTomahawkTwoTemp5(),
             AcctonAS7816_64XTomahawkTwoTemp6(),
             AcctonAS7816_64XTomahawkTwoTemp7(),
             AcctonAS7816_64XTomahawkTwoTemp8(),
             )
    f2b_temps = (AcctonAS7816_64XTemp1F2B(),
                 AcctonAS7816_64XTemp2F2B(),
                 AcctonAS7816_64XTemp3F2B(),
                 AcctonAS7816_64XTemp4F2B(),
                 AcctonAS7816_64XTemp5F2B(),
                 AcctonAS7816_64XTemp6F2B(),
                 )
    b2f_temps = (AcctonAS7816_64XTemp1B2F(),
                 AcctonAS7816_64XTemp2B2F(),
                 AcctonAS7816_64XTemp3B2F(),
                 AcctonAS7816_64XTemp4B2F(),
                 AcctonAS7816_64XTemp5B2F(),
                 AcctonAS7816_64XTemp6B2F(),
                 )
    if AcctonAS7816_64X_airflow() == 'B2F':
        temps = temps + b2f_temps
    else:
        temps = temps + f2b_temps

class AcctonAS7816_64XPWMs():
    units = (AcctonAS7816_64XPWM(),
             )

class AcctonAS7816_64XLEDs():
    units = (AcctonAS7816_64XSystemLED(),
             AcctonAS7816_64XFanLED(),
             AcctonAS7816_64XPSU1LED(),
             AcctonAS7816_64XPSU2LED(),
             )

class AcctonAS7816_64X(cumulus.platform.Platform):
    name = 'accton,as7816_64x'
    chip = cumulus.platform.TomahawkTwo_56970_Chip()
    switch = AcctonAS7816_64XSwitch(chip)
    cpld = AcctonAS7816_64XCPLD()
    sensors = AcctonAS7816_64XSensors()
    pwms = AcctonAS7816_64XPWMs()
    statusleds = AcctonAS7816_64XLEDs()

Platform = AcctonAS7816_64X
