#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class CelRedstoneSFPp(cumulus.platform.Trident10GPort):
    _set_led = '''
;;; CelRedstoneSFPp
;;; Two bits per port: yellow(active high), green(active high)
set_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        port 1
        pushst  PORTSTATUS_ZERO ; disable yellow LED
        pack

        ret
'''
    def __init__(self, label, serdes, leds):
        cumulus.platform.Trident10GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_type=None,
                                                 phy_addr=None,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 rx_lane_map=0x1032,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.hw_port_num(mode, 0))
        for led in led_index_list:
            # Increment the index by number of status bits per port
            set_calls[led] = call
            set_calls[led + 1] = None

        return set_calls

class CelRedstoneQSFPp(cumulus.platform.Trident40GPort):
    _set_led = '''
;;; CelRedstoneQSFPp
;;; Two bits per port: yellow(active high), green(active high)
set_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        port 1
        pushst  PORTSTATUS_ZERO ; disable yellow LED
        pack

        ret
'''
    _set_led_4x10 = '''
;;;
;;; Input : register A, logical port
;;; Input / Output : register B, accumulated port status

;;; For each port accumulate 2 bits of information in register B:
;;;   register B[0] - activity (TX _OR_ RX)
;;;   register B[1] - link enabled

SUB_INTF_ACTIVITY       equ     0x0  ; activity
SUB_INTF_LINKEN         equ     0x1  ; link enabled

sub_intf_info:

        ;;; push OR of current TX/RX activity onto stack
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor

        ;;; push accumlated link activity onto stack
        tst B, SUB_INTF_ACTIVITY
        push cy

        ;;; OR accumlated activity with current activity and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_ACTIVITY

        ;;; push current LINKEN onto stack
        pushst  PORTSTATUS_LINKEN

        ;;; push accumlated link enable onto stack
        tst B, SUB_INTF_LINKEN
        push cy

        ;;; OR accumlated LINKEN with current LINKEN and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_LINKEN

        ret

;;; CelRedstoneQSFPp_4x10
;;; OR together status of 4 ports for 4x10G QSFP port
;;; Two LED bits per port: yellow(active high), green(active low)
;;; Input: register A - first logical port of 4-port group
;;; Modified: register B - used as scratch space
set_led:

        ;;; initialize B
        ld      b,0

        ;;; load sub-0 info
        call    sub_intf_info

        ;;; load sub-1 info
        inc     a
        call    sub_intf_info

        ;;; load sub-2 info
        inc     a
        call    sub_intf_info

        ;;; load sub-3 info
        inc     a
        call    sub_intf_info

        ;;; push accumlated link activity onto stack
        tst     B, SUB_INTF_ACTIVITY
        push    cy

        ;;; combine with BLINK state
        push    (BLINK_STATE_P)
        tand
        tinv    ; flash off with activity

        ;;; push accumlated link enable onto stack
        tst     B, SUB_INTF_LINKEN
        push    cy

        ;;; AND activity-blink state with accumlated LINKEN
        tand

        ;;; LEDs are active low
        tinv
        pack

        port 1
        pushst  PORTSTATUS_ZERO ; disable yellow LED
        pack

        ret
'''
    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map, rx_polarity_flip=None, tx_polarity_flip=None):
        cumulus.platform.Trident40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=None,
                                                 phy_addrs=None,
                                                 leds=leds,
                                                 rx_lane_map=rx_lane_map,
                                                 rx_polarity_flip=rx_polarity_flip,
                                                 tx_lane_map=tx_lane_map,
                                                 tx_polarity_flip=tx_polarity_flip,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}
        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10,  self.led_port_num(mode, 0))
        elif mode == "40G":
            call = (fn_name, self._set_led, self.led_port_num(mode, 0))

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None

        return set_calls

class CelRedstoneLED(cumulus.ledup.TridentLEDController):
    num_leds = 52

class CelRedstoneSwitch(cumulus.platform.BCMSwitch):
    ledup = CelRedstoneLED
    _sfp = CelRedstoneSFPp
    _qsfp = CelRedstoneQSFPp

    has_phy_ucode_mdio = False # XXX unverified
    has_phy_ucode_spi_rom = False # XXX unverified

    ports = (
        _sfp(label="1",  serdes=12, leds=(0,  4)),
        _sfp(label="2",  serdes=13, leds=(0,  6)),
        _sfp(label="3",  serdes=14, leds=(0,  8)),
        _sfp(label="4",  serdes=15, leds=(0, 10)),
        _sfp(label="5",  serdes=16, leds=(0, 12)),
        _sfp(label="6",  serdes=17, leds=(0, 14)),
        _sfp(label="7",  serdes=18, leds=(0, 16)),
        _sfp(label="8",  serdes=19, leds=(0, 18)),
        _sfp(label="9",  serdes=20, leds=(0, 20)),
        _sfp(label="10", serdes=21, leds=(0, 22)),
        _sfp(label="11", serdes=22, leds=(0, 24)),
        _sfp(label="12", serdes=23, leds=(0, 26)),
        _sfp(label="13", serdes=24, leds=(0, 28)),
        _sfp(label="14", serdes=25, leds=(0, 30)),
        _sfp(label="15", serdes=26, leds=(0, 32)),
        _sfp(label="16", serdes=27, leds=(0, 34)),
        _sfp(label="17", serdes=28, leds=(0, 36)),
        _sfp(label="18", serdes=29, leds=(0, 38)),
        _sfp(label="19", serdes=30, leds=(0, 40)),
        _sfp(label="20", serdes=31, leds=(0, 42)),
        _sfp(label="21", serdes=32, leds=(0, 44)),
        _sfp(label="22", serdes=33, leds=(0, 46)),
        _sfp(label="23", serdes=34, leds=(0, 48)),
        _sfp(label="24", serdes=35, leds=(0, 50)),
        _sfp(label="25", serdes=36, leds=(1,  0)),
        _sfp(label="26", serdes=37, leds=(1,  2)),
        _sfp(label="27", serdes=38, leds=(1,  4)),
        _sfp(label="28", serdes=39, leds=(1,  6)),
        _sfp(label="29", serdes=40, leds=(1,  8)),
        _sfp(label="30", serdes=41, leds=(1, 10)),
        _sfp(label="31", serdes=42, leds=(1, 12)),
        _sfp(label="32", serdes=43, leds=(1, 14)),
        _sfp(label="33", serdes=44, leds=(1, 16)),
        _sfp(label="34", serdes=45, leds=(1, 18)),
        _sfp(label="35", serdes=46, leds=(1, 20)),
        _sfp(label="36", serdes=47, leds=(1, 22)),
        _sfp(label="37", serdes=48, leds=(1, 24)),
        _sfp(label="38", serdes=49, leds=(1, 26)),
        _sfp(label="39", serdes=50, leds=(1, 28)),
        _sfp(label="40", serdes=51, leds=(1, 30)),
        _sfp(label="41", serdes=56, leds=(1, 32)),
        _sfp(label="42", serdes=57, leds=(1, 34)),
        _sfp(label="43", serdes=58, leds=(1, 36)),
        _sfp(label="44", serdes=59, leds=(1, 38)),
        _sfp(label="45", serdes=60, leds=(1, 40)),
        _sfp(label="46", serdes=61, leds=(1, 42)),
        _sfp(label="47", serdes=62, leds=(1, 44)),
        _sfp(label="48", serdes=63, leds=(1, 46)),

        _qsfp(label="49", serdes=(64, 65, 66, 67), leds=(1, 48),
            rx_lane_map=0x0213, tx_lane_map=0x2031),
        _qsfp(label="50", serdes=(68, 69, 70, 71), leds=(1, 50),
            rx_lane_map=0x2031, tx_lane_map=0x3120,
            tx_polarity_flip=(True, True,False, False)),
        _qsfp(label="51", serdes=( 0,  1,  2,  3), leds=(0, 0),
          rx_lane_map=0x2031, rx_polarity_flip=(True, False, False, False),
          tx_lane_map=0x3120, tx_polarity_flip=(False, True,False, False)),
        _qsfp(label="52", serdes=( 4,  5,  6,  7), leds=(0, 2),
            rx_lane_map=0x1230, rx_polarity_flip=(True, False, False, True),
            tx_lane_map=0x3120, tx_polarity_flip=(True,True,False, False)),
    )

class CelRedstoneCPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/ffe05000.localbus/ffb00000.CPLD1'

class CelRedstonePSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class CelRedstonePSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

class CelRedstoneTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path = '/sys/devices/pci0000:00/0000:00:00.0/0000:01:00.0'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 100
    temp_crit = 105

class CelRedstoneTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Front Right Corner Ambient Temp Sensor'
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-004b'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 45
    temp_crit = 65

class CelRedstoneTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-004e'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 65
    temp_crit = 90

class CelRedstoneTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = 'P2020 Temp Sensor'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0048'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 65
    temp_crit = 90

class CelRedstoneTemp5(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    description = 'System Rear Outlet Temp Sensor'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0049'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 45
    temp_crit = 60

class CelRedstoneTemp6(cumulus.sensors.Temp_Unit):
    name = 'Temp6'
    description = 'Front Left Corner Ambient Temp Sensor'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004f'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 45
    temp_crit = 65

class CelRedstoneFan1(cumulus.sensors.Fan_Unit):
    name = 'Fan1'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004d'
    cpld_hwmon = ['fan1']
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']

class CelRedstoneFan2(cumulus.sensors.Fan_Unit):
    name = 'Fan2'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004d'
    cpld_hwmon = ['fan2']
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']

class CelRedstoneFan3(cumulus.sensors.Fan_Unit):
    name = 'Fan3'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004d'
    cpld_hwmon = ['fan3']
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']

class CelRedstoneFan4(cumulus.sensors.Fan_Unit):
    name = 'Fan4'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004d'
    cpld_hwmon = ['fan4']
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']

class CelRedstoneFan5(cumulus.sensors.Fan_Unit):
    name = 'Fan5'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004d'
    cpld_hwmon = ['fan5']
    driver_hwmon = ['fan5']
    pwm_hwmon = ['pwm5']

class CelRedstonePWM1(cumulus.sensors.PWM_Unit):
    fans = (CelRedstoneFan1(),
            CelRedstoneFan2(),
            CelRedstoneFan3(),
            CelRedstoneFan4(),
            CelRedstoneFan5(),
            )
    temps = (CelRedstoneTemp1(),
             CelRedstoneTemp2(),
             CelRedstoneTemp3(),
             CelRedstoneTemp4(),
             CelRedstoneTemp5(),
             CelRedstoneTemp6(),
             )

class CelRedstoneSensors():
    units = ( CelRedstonePSU1(),
              CelRedstonePSU2(),
              CelRedstoneFan1(),
              CelRedstoneFan2(),
              CelRedstoneFan3(),
              CelRedstoneFan4(),
              CelRedstoneFan5(),
              CelRedstoneTemp1(),
              CelRedstoneTemp2(),
              CelRedstoneTemp3(),
              CelRedstoneTemp4(),
              CelRedstoneTemp5(),
              CelRedstoneTemp6(),
              )

class CelRedstonePWMs():
    units = (CelRedstonePWM1(),
             )

class CelRedstoneSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_status'
    bad_led_color = 'green_blinking'

class CelRedstoneLEDs():
    units = (CelRedstoneSystemLED(),
             )

class CelRedstone(cumulus.platform.Platform):
    name = 'cel,redstone'
    chip = cumulus.platform.TridentPlusChip()
    switch = CelRedstoneSwitch(chip)
    cpld = CelRedstoneCPLD()
    statusleds = CelRedstoneLEDs()
    pwms = CelRedstonePWMs()
    sensors = CelRedstoneSensors()

Platform = CelRedstone
