#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class DellZ9100MgmtSFP(cumulus.platform.TomahawkMgmt10GPort):
    def __init__(self, label, serdes, leds):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')
        cumulus.platform.TomahawkMgmt10GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=None,
                                                   rx_polarity_flip=None,
                                                   tx_lane_map=None,
                                                   tx_polarity_flip=None,
                                                   pre_emphasis=None,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    _set_led_mgmt = '''
;;;
;;; A single bi-color LED is used to indicate link/activity on the
;;; management ports
;;;
set_led:
        call    set_port_led
        ret
'''

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        set_calls[led_index_list[0]] = (fn_name, self._set_led_mgmt, led_port)
        set_calls[led_index_list[0]+1] = None

        return set_calls

class DellZ9100QSFP28(cumulus.platform.Tomahawk100GPort):
    '''
        Each port has eight bits of LED data controlling four bi-color green/amber
        LEDs. The LEDs behave as follows:
          00 - Green on
          01 - Amber on
          10 - Amber flash
          11 - both off
    '''
    _set_led_common = '''
;;;
;;; common LED routines
;;;
set_port_led:
; pack two bits
;  00 => green
;  11 => off
        ld      b, (ID_ENABLE_P)
        tst     b, ID_ENABLE_BIT
        jc      id_mode

        port  a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                       ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        jmp     pack_it

set_led_off:
;  11 => off
        push    1
        jmp     pack_it

id_mode:
        push    1
        cmp     a, (ID_PORT_P)
        jnz     pack_it
        ld      b, (WIGGLE_STATE_P)
        and     b, (WAG_STATE_P)
        push    b
        tinv
pack_it:
; pack two copies of T
        pop
        push    cy
        pack
        push    cy
        pack
        ret
'''

    _set_led_1x = '''
;;;
;;; In 1x mode, only the green LED of the first bi-color LED is used to
;;; indicate link/activity. The rest are off.
;;;
set_led:
        call    set_port_led
        call    set_led_off
        call    set_led_off
        call    set_led_off

        ret
'''

    _set_led_2x = '''
;;;
;;; In 2x mode, we use the green LED of the first and second bi-color LED to
;;; indicate link/activity. The rest are off.
;;;
set_led:
        call    set_port_led
        inc     a
        inc     a
        call    set_port_led
        call    set_led_off
        call    set_led_off

        ret
'''

    _set_led_4x = '''
;;;
;;; In 4x mode, the green LEDs of all four bi-color LEDs are used to
;;; indicate link/activity.
;;;
set_led:
        call    set_port_led
        inc     a
        call    set_port_led
        inc     a
        call    set_port_led
        inc     a
        call    set_port_led
        ret
'''

    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None

        return set_calls

class DellZ9100LED(cumulus.ledup.TomahawkLEDController):
    led_functions = (DellZ9100QSFP28._set_led_common)
    num_leds = 128
    has_phys_id = True

class DellZ9100Switch(cumulus.platform.BCMSwitch):
    ledup = DellZ9100LED
    led_controllers = set([0, 1, 2])
    _mgmtSfp = DellZ9100MgmtSFP
    _qsfp28 = DellZ9100QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp28(label="1", serdes=( 48,  49,  50,  51), leds=(1, 0),
              tx_polarity_flip=0x000e, rx_polarity_flip=0x000c,
              tx_lane_map=0x0132, rx_lane_map=0x1023),
        _qsfp28(label="2", serdes=( 52,  53,  54,  55), leds=(1, 8),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x000d,
              tx_lane_map=0x2301, rx_lane_map=0x1302),
        _qsfp28(label="3", serdes=( 56,  57,  58,  59), leds=(1, 16),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x0002,
              tx_lane_map=0x0123, rx_lane_map=0x1203),
        _qsfp28(label="4", serdes=( 60,  61,  62,  63), leds=(1,  24),
              tx_polarity_flip=0x000b, rx_polarity_flip=0x000f,
              tx_lane_map=0x3201, rx_lane_map=0x1302),
        _qsfp28(label="5", serdes=( 64,  65,  66,  67), leds=(1, 32),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x000c,
              tx_lane_map=0x3210, rx_lane_map=0x3201),
        _qsfp28(label="6", serdes=( 68,  69,  70,  71), leds=(1, 40),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0002,
              tx_lane_map=0x2301, rx_lane_map=0x1302),
        _qsfp28(label="7", serdes=( 72,  73,  74,  75), leds=(1, 48),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0002,
              tx_lane_map=0x0123, rx_lane_map=0x1203),
        _qsfp28(label="8", serdes=( 76,  77,  78,  79), leds=(1, 56),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x000d,
              tx_lane_map=0x1320, rx_lane_map=0x2301),
        _qsfp28(label="9", serdes=( 36,  37,  38,  39), leds=(1, 64),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000d,
              tx_lane_map=0x1032, rx_lane_map=0x0312),
        _qsfp28(label="10", serdes=( 32,  33,  34,  35), leds=(1, 72),
              tx_polarity_flip=0x000b, rx_polarity_flip=0x000a,
              tx_lane_map=0x2031, rx_lane_map=0x1302),
        _qsfp28(label="11", serdes=( 44,  45,  46,  47), leds=(1, 80),
              tx_polarity_flip=0x000e, rx_polarity_flip=0x0007,
              tx_lane_map=0x1023, rx_lane_map=0x2103),
        _qsfp28(label="12", serdes=( 40,  41,  42,  43), leds=(1, 88),
              tx_polarity_flip=0x0006, rx_polarity_flip=0x000f,
              tx_lane_map=0x0132, rx_lane_map=0x3210),
        _qsfp28(label="13", serdes=( 80,  81,  82,  83), leds=(1, 96),
              tx_polarity_flip=0x0006, rx_polarity_flip=0x000f,
              tx_lane_map=0x0213, rx_lane_map=0x2301),
        _qsfp28(label="14", serdes=( 84,  85,  86,  87), leds=(1, 104),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000d,
              tx_lane_map=0x1032, rx_lane_map=0x0213),
        _qsfp28(label="15", serdes=( 88,  89,  90,  91), leds=(1, 112),
              tx_polarity_flip=0x0005, rx_polarity_flip=0x0004,
              tx_lane_map=0x0132, rx_lane_map=0x3210),
        _qsfp28(label="16", serdes=( 92,  93,  94,  95), leds=(1, 120),
              tx_polarity_flip=0x0008, rx_polarity_flip=0x000b,
              tx_lane_map=0x0123, rx_lane_map=0x3210),
        _qsfp28(label="17", serdes=( 96,  97,  98,  99), leds=(0, 0),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0002,
              tx_lane_map=0x0123, rx_lane_map=0x3201),
        _qsfp28(label="18", serdes=(100, 101, 102, 103), leds=(0, 8),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000d,
              tx_lane_map=0x0123, rx_lane_map=0x0213),
        _qsfp28(label="19", serdes=(104, 105, 106, 107), leds=(0, 16),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x1032, rx_lane_map=0x3210),
        _qsfp28(label="20", serdes=(108, 109, 110, 111), leds=(0, 24),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x3210),
        _qsfp28(label="21", serdes=( 20,  21,  22,  23), leds=(0, 32),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x2301, rx_lane_map=0x3102),
        _qsfp28(label="22", serdes=( 16,  17,  18,  19), leds=(0, 40),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0005,
              tx_lane_map=0x3102, rx_lane_map=0x2103),
        _qsfp28(label="23", serdes=( 28,  29,  30,  31), leds=(0, 48),
              tx_polarity_flip=0x000b, rx_polarity_flip=0x0000,
              tx_lane_map=0x1023, rx_lane_map=0x2301),
        _qsfp28(label="24", serdes=( 24,  25,  26,  27), leds=(0, 56),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x0002,
              tx_lane_map=0x2130, rx_lane_map=0x2310),
        _qsfp28(label="25", serdes=( 116, 117, 118, 119), leds=(0, 64),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000c,
              tx_lane_map=0x2310, rx_lane_map=0x3201),
        _qsfp28(label="26", serdes=( 112, 113, 114, 115), leds=(0, 72),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0008,
              tx_lane_map=0x2013, rx_lane_map=0x0123),
        _qsfp28(label="27", serdes=( 124, 125, 126, 127), leds=(0, 80),
              tx_polarity_flip=0x000b, rx_polarity_flip=0x0004,
              tx_lane_map=0x0132, rx_lane_map=0x1023),
        _qsfp28(label="28", serdes=( 120, 121, 122, 123), leds=(0, 88),
              tx_polarity_flip=0x0007, rx_polarity_flip=0x0007,
              tx_lane_map=0x0123, rx_lane_map=0x0213),
        _qsfp28(label="29", serdes=(   4,   5,   6,   7), leds=(0, 96),
              tx_polarity_flip=0x0008, rx_polarity_flip=0x0007,
              tx_lane_map=0x0213, rx_lane_map=0x1203),
        _qsfp28(label="30", serdes=(   0,   1,   2,   3), leds=(0, 104),
              tx_polarity_flip=0x0003, rx_polarity_flip=0x0009,
              tx_lane_map=0x0123, rx_lane_map=0x0213),
        _qsfp28(label="31", serdes=(  12,  13,  14,  15), leds=(0, 112),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x000c,
              tx_lane_map=0x2301, rx_lane_map=0x3201),
        _qsfp28(label="32", serdes=(   8,   9,  10,  11), leds=(0, 120),
              tx_polarity_flip=0x0003, rx_polarity_flip=0x0008,
              tx_lane_map=0x0123, rx_lane_map=0x0213),
        _mgmtSfp(label="33", serdes=(128,), leds=(2,124)),
        _mgmtSfp(label="34", serdes=(130,), leds=(2,126)),

    )

class DellZ9100CPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/platform/dell_z9100_smf.0/hwmon'
    if os.path.exists(path): 
        files = [ f for f in os.listdir(path) if f.startswith('hwmon') ]
        if len(files):
            path = os.path.join(path, files[0])
        
class DellZ9100DIMMTemp(cumulus.sensors.Temp_Unit):
    name = 'DIMMTemp'
    description = cumulus.sensors.Temp_Unit.dimm_sensor_name
    driver_hwmon = ['temp1']
    temp_fan_min = None
    temp_fan_max = None
    temp_max = 90
    temp_crit = 95

    driver_path = ''
    pci_path = '/sys/devices/pci0000:00/0000:00:1f.3'
    if os.path.exists(pci_path):
        for x in os.listdir(pci_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                c = b + '-0018'
                driver_path = os.path.join(pci_path, x, c)
                break
            
class DellZ9100CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'

    temp_fan_min = None
    temp_fan_max = None
    temp_otp_hw = True
    temp_max = 90
    temp_crit = 98

class DellZ9100CT1(DellZ9100CoreTemp):
    name = 'Temp1'
    driver_hwmon = ['temp2']

class DellZ9100CT2(DellZ9100CoreTemp):
    name = 'Temp2'
    driver_hwmon = ['temp3']

class DellZ9100CT3(DellZ9100CoreTemp):
    name = 'Temp3'
    driver_hwmon = ['temp4']

class DellZ9100CT4(DellZ9100CoreTemp):
    name = 'Temp4'
    driver_hwmon = ['temp5']

class DellZ9100SMFTemp(cumulus.sensors.Temp_Unit):
    driver_path = '/sys/devices/platform/dell_z9100_smf.0'    
    temp_fan_min = None
    temp_fan_max = None

class DellZ9100SMFTemp1(DellZ9100SMFTemp):
    # U2900 - CPU On- board sensor
    # F2B And B2F
    name = 'SMFTemp1'
    description = 'CPU on-board Temp Sensor'
    driver_hwmon = ['temp1']
    temp_otp_hw = True
    temp_max = 100
    temp_crit = 125
    temp_otp_hw = True

class DellZ9100SMFTemp2(DellZ9100SMFTemp):
    # U44 BCM56960 on-board 2
    # F2B
    name = 'SMFTemp2'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_hwmon = ['temp2']
    temp_otp_hw = True
    temp_max = 85
    temp_crit = 95

class DellZ9100SMFTemp3(DellZ9100SMFTemp):
    # U4 - system inlet 1(for front to rear airflow path)
    # F2B And B2F
    name = 'SMFTemp3'
    description = 'Front Panel (Left) Temp Sensor'
    driver_hwmon = ['temp3']

class DellZ9100SMFTemp3F2B(DellZ9100SMFTemp3):
    temp_max = 50

class DellZ9100SMFTemp3B2F(DellZ9100SMFTemp3):
    temp_max = 55
                
class DellZ9100SMFTemp4(DellZ9100SMFTemp):
    # U2 system inlet 2(for front to rear airflow path)
    # F2B and B2F
    name = 'SMFTemp4'
    description = 'Front Panel (Right) Temp Sensor'
    driver_hwmon = ['temp4']

class DellZ9100SMFTemp4F2B(DellZ9100SMFTemp4):
    temp_max = 50

class DellZ9100SMFTemp4B2F(DellZ9100SMFTemp4):
    temp_max = 55

class DellZ9100SMFTemp5(DellZ9100SMFTemp):
    # U38 BCM56960 on-board 1.
    # B2F
    name = 'SMFTemp5'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_hwmon = ['temp6']
    temp_otp_hw = True
    temp_max = 90
    temp_crit = 95

class DellZ9100SMFTemp6(DellZ9100SMFTemp):
    # U2900 System inlet for rear to front airflow path
    # B2F
    name = 'SMFTemp6'
    description = "Rear Temp Sensor"
    driver_hwmon = ['temp9']
    temp_max = 50

class DellZ9100SMFTemp7(DellZ9100SMFTemp):
    name = 'PSU1Temp'
    driver_hwmon = ['temp11']
    psu = 'PSU1'

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class DellZ9100SMFTemp8(DellZ9100SMFTemp):
    name = 'PSU2Temp'
    driver_hwmon = ['temp12']
    psu = 'PSU2'

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

# Z9100, tomahawk internal temp sensors
class DellZ9100THTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 105
    temp_crit    = 110
    temp_fan_min = None
    temp_fan_max = None

class DellZ9100THTemp1(DellZ9100THTemp):
    name = 'Temp5'
    driver_hwmon = ['temp1']

class DellZ9100THTemp2(DellZ9100THTemp):
    name = 'Temp6'
    driver_hwmon = ['temp2']

class DellZ9100THTemp3(DellZ9100THTemp):
    name = 'Temp7'
    driver_hwmon = ['temp3']

class DellZ9100THTemp4(DellZ9100THTemp):
    name = 'Temp8'
    driver_hwmon = ['temp4']

class DellZ9100THTemp5(DellZ9100THTemp):
    name = 'Temp9'
    driver_hwmon = ['temp5']

class DellZ9100THTemp6(DellZ9100THTemp):
    name = 'Temp10'
    driver_hwmon = ['temp6']

class DellZ9100THTemp7(DellZ9100THTemp):
    name = 'Temp11'
    driver_hwmon = ['temp7']

class DellZ9100THTemp8(DellZ9100THTemp):
    name = 'Temp12'
    driver_hwmon = ['temp8']

class DellZ9100SMFFan(cumulus.sensors.Fan_Unit):
    rpmmode = 1
    fan_max = 24000
    fan_min = 5000
    driver_path = '/sys/devices/platform/dell_z9100_smf.0'

class DellZ9100SMFFan1(DellZ9100SMFFan):
    name = 'Fan1'
    description = 'Fan Tray 1, Fan 1'
    driver_hwmon = ['fan1']
    cpld_hwmon = ['fan1']

class DellZ9100SMFFan2(DellZ9100SMFFan):
    name = 'Fan2'
    description = 'Fan Tray 1, Fan 2'
    driver_hwmon = ['fan2']
    cpld_hwmon = ['fan2']

class DellZ9100SMFFan3(DellZ9100SMFFan):
    name = 'Fan3'
    description = 'Fan Tray 2, Fan 1'
    driver_hwmon = ['fan3']
    cpld_hwmon = ['fan3']

class DellZ9100SMFFan4(DellZ9100SMFFan):
    name = 'Fan4'
    description = 'Fan Tray 2, Fan 2'
    driver_hwmon = ['fan4']
    cpld_hwmon = ['fan4']

class DellZ9100SMFFan5(DellZ9100SMFFan):
    name = 'Fan5'
    description = 'Fan Tray 3, Fan 1'
    driver_hwmon = ['fan5']
    cpld_hwmon = ['fan5']

class DellZ9100SMFFan6(DellZ9100SMFFan):
    name = 'Fan6'
    description = 'Fan Tray 3, Fan 2'
    driver_hwmon = ['fan6']
    cpld_hwmon = ['fan6']

class DellZ9100SMFFan7(DellZ9100SMFFan):
    name = 'Fan7'
    description = 'Fan Tray 4, Fan 1'
    driver_hwmon = ['fan7']
    cpld_hwmon = ['fan7']

class DellZ9100SMFFan8(DellZ9100SMFFan):
    name = 'Fan8'
    description = 'Fan Tray 4, Fan 2'
    driver_hwmon = ['fan8']
    cpld_hwmon = ['fan8']

class DellZ9100SMFFan9(DellZ9100SMFFan):
    name = 'Fan9'
    description = 'Fan Tray 5, Fan 1'
    driver_hwmon = ['fan9']
    cpld_hwmon = ['fan9']

class DellZ9100SMFFan10(DellZ9100SMFFan):
    name = 'Fan10'
    description = 'Fan Tray 5, Fan 2'
    driver_hwmon = ['fan10']
    cpld_hwmon = ['fan10']

class DellZ9100SMFFan11(DellZ9100SMFFan):
    name = 'PSU1Fan'
    driver_hwmon = ['fan11']
    cpld_hwmon = ['fan11']
    psu = 'PSU1'

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)


class DellZ9100SMFFan12(DellZ9100SMFFan):
    name = 'PSU2Fan'
    driver_hwmon = ['fan12']
    cpld_hwmon = ['fan12']
    psu = 'PSU2'

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class DellZ9100SMFPSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    driver_path = '/sys/devices/platform/dell_z9100_smf.0/hwmon'
    if os.path.exists(driver_path): 
        files = [ f for f in os.listdir(driver_path) if f.startswith('hwmon') ]
        if len(files):
            driver_path = os.path.join(driver_path, files[0])
    cpld_path = driver_path            

    cpld_hwmon = ['psu_pwr1']
    driver_hwmon = ['power1']

class DellZ9100SMFPSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    driver_path = '/sys/devices/platform/dell_z9100_smf.0/hwmon'
    if os.path.exists(driver_path): 
        files = [ f for f in os.listdir(driver_path) if f.startswith('hwmon') ]
        if len(files):
            driver_path = os.path.join(driver_path, files[0])
    cpld_path = driver_path            

    cpld_hwmon = ['psu_pwr2']
    driver_hwmon = ['power2']

class DellZ9100PWM(cumulus.sensors.PWM_Unit):
    def __init__(self):
        pass
    def run_state(self):
        pass

class DellZ9100LED(cumulus.sensors.StatusLED_Unit):
    def __init__(self):
        pass

    def run_state(self):
        pass

def dell_z9100_airflow():
    b2f_part_nums = ['09N7MF', '0GTX3X', '0KF3Y8', '0J86XN']
    f2b_part_nums = ['04HXVK', '03GT5N', '0Y72TP', '062YMV']

    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return None
    if plat.strip() != 'dell,z9100_c2538':
        return None

    try:
        b_info = subprocess.check_output('/usr/cumulus/bin/decode-syseeprom')
    except:
        b_info = ''

    # Part Number Format
    # TLV Name             Code Len Value
    # -------------------- ---- --- -----
    # Part Number          0x22   6 0J09D3
    part_num = ''    
    for l in b_info.split('\n'):
        if 'Part Number' in l:
            part_num = l.split()[4]
            break
    if part_num in b2f_part_nums:
        return 'B2F'        
    if part_num in f2b_part_nums:
        return 'F2B'        
    return None
    
class DellZ9100Sensors():
    
    units = (DellZ9100SMFTemp1(),
             DellZ9100SMFTemp7(),
             DellZ9100SMFTemp8(),
             DellZ9100SMFFan1(),
             DellZ9100SMFFan2(),
             DellZ9100SMFFan3(),
             DellZ9100SMFFan4(),
             DellZ9100SMFFan5(),
             DellZ9100SMFFan6(),
             DellZ9100SMFFan7(),
             DellZ9100SMFFan8(),
             DellZ9100SMFFan9(),
             DellZ9100SMFFan10(),
             DellZ9100SMFFan11(),
             DellZ9100SMFFan12(),
             DellZ9100SMFPSU1(),
             DellZ9100SMFPSU2(),
             DellZ9100DIMMTemp(),
             DellZ9100CT1(),
             DellZ9100CT2(),
             DellZ9100CT3(),
             DellZ9100CT4(),
             DellZ9100THTemp1(),
             DellZ9100THTemp2(),
             DellZ9100THTemp3(),
             DellZ9100THTemp4(),
             DellZ9100THTemp5(),
             DellZ9100THTemp6(),
             DellZ9100THTemp7(),
             DellZ9100THTemp8(),
            )
    f2b_temps = (DellZ9100SMFTemp2(),
                 DellZ9100SMFTemp3F2B(),
                 DellZ9100SMFTemp4F2B(),
        )
    b2f_temps = (DellZ9100SMFTemp3B2F(),
                 DellZ9100SMFTemp4B2F(),
                 DellZ9100SMFTemp5(),
                 DellZ9100SMFTemp6(),                     
    )
    if dell_z9100_airflow() == 'F2B':
        units = units + f2b_temps
    else:
        units = units + b2f_temps

                    
class DellZ9100(cumulus.platform.Platform):
    name = 'dell,z9100_c2538'
    chip = cumulus.platform.TomahawkChip()
    switch = DellZ9100Switch(chip)
    cpld = DellZ9100CPLD()
    statusleds = None
    pwms = None
    sensors = DellZ9100Sensors()

Platform = DellZ9100
