#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class FBWedge_16QSFPp(cumulus.platform.TridentTwo40GPort):
	'''
;;; Facebook Wedge_16
;;;
;;; (Notes adapted from Adam Simpkins post on the Facebook-Cumulus facebook
;;; page)
;;;
;;; Each LED microprocessor controls half of the ports. The first
;;; microprocessor controls the left 8 QSFPs, and the second controls the
;;; right 8 QSFPs.
;;;
;;; The bitstream consists of two bits per (10G) port, making for 64 bits
;;; from each microprocessor. Each LED will be on if the bits are (10b),
;;; and off for any other combination (00b, 01b, 11b).
;;;
;;; The microprocessors always send data for all ports. The FPGA decides
;;; which port LEDs to display based on the toggle button on the front
;;; panel. There are only four LEDs per pair of QSFP+ ports.  The toggle
;;; button switches between displaying the top versus the bottom ports.
;;;
;;; When a port is in 10G mode, all four blue LEDs are used. When a port
;;; is in 40G mode, only the outer two LEDs are used.
'''
	_set_led = '''
'''

	_set_led_40G_top = '''
;;; FBWedge_16XQSFPp_top
qsfp40t_set_led_blue:
        port    a
	pushst  PORTSTATUS_RX
	pushst  PORTSTATUS_TX
	tor
	push    (BLINK_STATE_P)
	tand
	tinv                    ; flash off with activity
	pushst  PORTSTATUS_LINKEN
	tand
	pack
	pushst  PORTSTATUS_ZERO
	pack
	ret
qsfp40t_set_led_off:
        port    1
	pushst  PORTSTATUS_ZERO
	pack
	pushst  PORTSTATUS_ZERO
	pack
	ret
set_led:
	call    qsfp40t_set_led_blue
	call    qsfp40t_set_led_off
	call    qsfp40t_set_led_off
	call    qsfp40t_set_led_off
	ret
'''
	_set_led_40G_bot = '''
;;; FBWedge_16XQSFPp_bot
qsfp40b_set_led_blue:
        port    a
	pushst  PORTSTATUS_RX
	pushst  PORTSTATUS_TX
	tor
	push    (BLINK_STATE_P)
	tand
	tinv                    ; flash off with activity
	pushst  PORTSTATUS_LINKEN
	tand
	pack
	pushst  PORTSTATUS_ZERO
	pack
	ret
qsfp40b_set_led_off:
        port    1
	pushst  PORTSTATUS_ZERO
	pack
	pushst  PORTSTATUS_ZERO
	pack
	ret
set_led:
	call    qsfp40b_set_led_off
	call    qsfp40b_set_led_off
	call    qsfp40b_set_led_off
	call    qsfp40b_set_led_blue
	ret
'''
	_set_led_4x10 = '''
;;; FBWedge_16XQSFPp_4x10
qsfp10_set_led_blue:
        port    a
	pushst  PORTSTATUS_RX
	pushst  PORTSTATUS_TX
	tor
	push    (BLINK_STATE_P)
	tand
	tinv                    ; flash off with activity
	pushst  PORTSTATUS_LINKEN
	tand
	pack
	pushst  PORTSTATUS_ZERO
	pack
	ret
qsfp10_set_led_off:
        port    1
	pushst  PORTSTATUS_ZERO
	pack
	pushst  PORTSTATUS_ZERO
	pack
	ret
set_led:
        call    qsfp10_set_led_blue
	inc     a
        call    qsfp10_set_led_blue
	inc     a
        call    qsfp10_set_led_blue
	inc     a
        call    qsfp10_set_led_blue
	ret
'''

	def __init__(self, label, serdes, leds, pre_emphasis,
		     rx_lane_map, tx_lane_map,
		     rx_polarity_flip=None, tx_polarity_flip=None,
		     drv_current=None, pre_drv_current=None):
		cumulus.platform.TridentTwo40GPort.__init__(self,
							    label=label,
							    serdes=serdes,
							    phy_types=None,
							    phy_addrs=None,
							    phy_layer='QSFP+',
							    leds=leds,
							    rx_lane_map=rx_lane_map,
							    rx_polarity_flip=rx_polarity_flip,
							    tx_lane_map=tx_lane_map,
							    tx_polarity_flip=tx_polarity_flip,
							    pre_emphasis=pre_emphasis,
							    drv_current=drv_current,
							    pre_drv_current=pre_drv_current,
							    oversub=True,
							    unit=0)
		self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')

	def set_leds(self, mode, led_port, sub, led_index_list):
		self._check_mode(mode)

		set_calls = {}

		if (mode == "4x10G") and (sub != 0):
			# For 4x10G mode only update LEDs for the first
			# sub-interface.
			return set_calls

		if mode == "4x10G":
			call = ("FBWedge_16QSFP_4x10", self._set_led_4x10, led_port)
		elif mode == "40G":
			if self._is_even_port():
				call = ("FBWedge_QSFP_40G_top", self._set_led_40G_top, led_port)
			else:
				call = ("FBWedge_QSFP_40G_bot", self._set_led_40G_bot, led_port)

		led_index = led_index_list[0]
		set_calls[led_index] = call
		set_calls[led_index + 1] = None
		set_calls[led_index + 2] = None
		set_calls[led_index + 3] = None
		set_calls[led_index + 4] = None
		set_calls[led_index + 5] = None
		set_calls[led_index + 6] = None
		set_calls[led_index + 7] = None

		return set_calls

	def _is_even_port(self):
		return (self.leds[1] / 8) & 1


class FBWedge_16LED(cumulus.ledup.TridentTwoLEDController):
	num_leds = 64


class FBWedge_16Switch(cumulus.platform.BCMSwitch):
	ledup = FBWedge_16LED
	_qsfp = FBWedge_16QSFPp

	has_phy_ucode_mdio = False
	has_phy_ucode_spi_rom = False

	ports = (
		_qsfp(label="1", serdes=(32, 33, 34, 35), leds=(0, 0),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x1230),
		_qsfp(label="2", serdes=(36, 37, 38, 39), leds=(0, 8),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x3210),
		_qsfp(label="3", serdes=(40, 41, 42, 43), leds=(0, 16),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x1230),
		_qsfp(label="4", serdes=(44, 45, 46, 47), leds=(0, 24),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x3210),
		_qsfp(label="5", serdes=(48, 49, 50, 51), leds=(0, 32),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x1230),
		_qsfp(label="6", serdes=(52, 53, 54, 55), leds=(0, 40),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x3210),
		_qsfp(label="7", serdes=(56, 57, 58, 59), leds=(0, 48),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x1230),
		_qsfp(label="8", serdes=(60, 61, 62, 63), leds=(0, 56),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x3210),
		_qsfp(label="9", serdes=(64, 65, 66, 67), leds=(1, 0),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x3210),
		_qsfp(label="10", serdes=(68, 69, 70, 71), leds=(1, 8),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x3210),
		_qsfp(label="11", serdes=(72, 73, 74, 75), leds=(1, 16),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x1230),
		_qsfp(label="12", serdes=(76, 77, 78, 79), leds=(1, 24),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x3210),
		_qsfp(label="13", serdes=(80, 81, 82, 83), leds=(1, 32),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x1230),
		_qsfp(label="14", serdes=(84, 85, 86, 87), leds=(1, 40),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x3210),
		_qsfp(label="15", serdes=(88, 89, 90, 91), leds=(1, 48),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x1230),
		_qsfp(label="16", serdes=(92, 93, 94, 95), leds=(1, 56),
		      pre_emphasis=(0xd2b0, 0xd2b0, 0xd2b0, 0xd2b0),
		      rx_lane_map=0x3012, tx_lane_map=0x3210),
		)

class FBWedge_16(cumulus.platform.Platform):
	name = 'facebook,wedge'
	chip = cumulus.platform.TridentTwo_56850_Chip()
	switch = FBWedge_16Switch(chip)

Platform = FBWedge_16
