#
# Copyright (c) 2018 Cumulus Networks, Inc.  All Rights Reserved.
#

import subprocess
import syslog
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

# 10/100/1000 port LEDs are driven by the BCM54282 PHYs.  The PHY uses the
# LED[2] pins for Link/Activity and LED[0] pins for speed indication.

class LenovoNE0152TGigE(cumulus.platform.Helix41GPort):
    def __init__(self, label, hw_port, phy_addr, prim_offset):
        cumulus.platform.Helix41GPort.__init__(self,
                                               label=label,
                                               hw_port=hw_port,
                                               phy_type="54282",
                                               phy_addr=phy_addr,
                                               phy_layer="1GBaseT",
                                               prim_offset=prim_offset,
                                               leds=None,
                                               unit=0)

class LenovoNE0152TSFPp(cumulus.platform.Helix410GPort):
    '''
;;; Lenovo NE0152T
;;;
;;; There are two LEDs for each of the four SFP+ ports.  The two bits of
;;; LED data operate as follows (the first bit lights the right LED and
;;; the second bit lights the left LED):
;;;
;;;       2nd bit       1st bit
;;;      +-------+     +-------+
;;;    0 | green |     | green |
;;;      +-------+     +-------+
;;;    1 |  off  |     |  off  |
;;;      +-------+     +-------+
;;;
;;; The first bit will be used to indicate Link, and the second bit will
;;; be used it indicate activity.
;;;
;;; '''
    _set_led = '''
set_led:
        port a

        pushst  PORTSTATUS_LINKEN
        tinv
        pack

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv
        pack

        ret
'''
    def __init__(self, label, hw_port, phy_addr, leds, rx_polarity_flip=None,
                 tx_polarity_flip=None, tx_lane_map=None, rx_lane_map=None):
        cumulus.platform.Helix410GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type="84758",
                                                phy_addr=phy_addr,
                                                phy_layer="SFP+",
                                                leds=leds,
                                                rx_lane_map=rx_lane_map,
                                                tx_lane_map=tx_lane_map,
                                                rx_polarity_flip=rx_polarity_flip,
                                                tx_polarity_flip=tx_polarity_flip,
                                                unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, 0)

        call = (fn_name, self._set_led, led_port)

        led_index = led_index_list[0]
        set_calls[led_index] = call
        set_calls[led_index + 1] = None

        return set_calls

                                                                                                
class LenovoNE0152TLED(cumulus.ledup.Helix4LEDController):
    num_leds = 8

# Switch Ports

class LenovoNE0152TSwitch(cumulus.platform.BCMSwitch):
    ledup = LenovoNE0152TLED
    _gige = LenovoNE0152TGigE
    _sfp = LenovoNE0152TSFPp
    has_phy_ucode_mdio = True
    has_phy_ucode_spi_rom = False

    ports = (
        _gige(label="1",  hw_port='ge25', phy_addr=0x22, prim_offset=0x1901),
        _gige(label="2",  hw_port='ge24', phy_addr=0x21, prim_offset=0x1900),
        _gige(label="3",  hw_port='ge26', phy_addr=0x23, prim_offset=0x1902),
        _gige(label="4",  hw_port='ge27', phy_addr=0x24, prim_offset=0x1903),
        _gige(label="5",  hw_port='ge29', phy_addr=0x26, prim_offset=0x1905),
        _gige(label="6",  hw_port='ge28', phy_addr=0x25, prim_offset=0x1904),
        _gige(label="7",  hw_port='ge30', phy_addr=0x27, prim_offset=0x1906),
        _gige(label="8",  hw_port='ge31', phy_addr=0x28, prim_offset=0x1907),

        _gige(label="9",  hw_port='ge33', phy_addr=0x2c, prim_offset=0x2101),
        _gige(label="10", hw_port='ge32', phy_addr=0x2b, prim_offset=0x2100),
        _gige(label="11", hw_port='ge34', phy_addr=0x2d, prim_offset=0x2102),
        _gige(label="12", hw_port='ge35', phy_addr=0x2e, prim_offset=0x2103),
        _gige(label="13", hw_port='ge37', phy_addr=0x30, prim_offset=0x2105),
        _gige(label="14", hw_port='ge36', phy_addr=0x2f, prim_offset=0x2104),
        _gige(label="15", hw_port='ge38', phy_addr=0x31, prim_offset=0x2106),
        _gige(label="16", hw_port='ge39', phy_addr=0x32, prim_offset=0x2107),

        _gige(label="17", hw_port='ge41', phy_addr=0x36, prim_offset=0x2901),
        _gige(label="18", hw_port='ge40', phy_addr=0x35, prim_offset=0x2900),
        _gige(label="19", hw_port='ge42', phy_addr=0x37, prim_offset=0x2902),
        _gige(label="20", hw_port='ge43', phy_addr=0x38, prim_offset=0x2903),
        _gige(label="21", hw_port='ge45', phy_addr=0x3a, prim_offset=0x2905),
        _gige(label="22", hw_port='ge44', phy_addr=0x39, prim_offset=0x2904),
        _gige(label="23", hw_port='ge46', phy_addr=0x3b, prim_offset=0x2906),
        _gige(label="24", hw_port='ge47', phy_addr=0x3c, prim_offset=0x2907),

        _gige(label="25", hw_port='ge1',  phy_addr=0x02, prim_offset=0x0101),
        _gige(label="26", hw_port='ge0',  phy_addr=0x01, prim_offset=0x0100),
        _gige(label="27", hw_port='ge2',  phy_addr=0x03, prim_offset=0x0102),
        _gige(label="28", hw_port='ge3',  phy_addr=0x04, prim_offset=0x0103),
        _gige(label="29", hw_port='ge5',  phy_addr=0x06, prim_offset=0x0105),
        _gige(label="30", hw_port='ge4',  phy_addr=0x05, prim_offset=0x0104),
        _gige(label="31", hw_port='ge6',  phy_addr=0x07, prim_offset=0x0106),
        _gige(label="32", hw_port='ge7',  phy_addr=0x08, prim_offset=0x0107),

        _gige(label="33", hw_port='ge9',  phy_addr=0x0c, prim_offset=0x0901),
        _gige(label="34", hw_port='ge8',  phy_addr=0x0b, prim_offset=0x0900),
        _gige(label="35", hw_port='ge10', phy_addr=0x0d, prim_offset=0x0902),
        _gige(label="36", hw_port='ge11', phy_addr=0x0e, prim_offset=0x0903),
        _gige(label="37", hw_port='ge13', phy_addr=0x10, prim_offset=0x0905),
        _gige(label="38", hw_port='ge12', phy_addr=0x0f, prim_offset=0x0904),
        _gige(label="39", hw_port='ge14', phy_addr=0x11, prim_offset=0x0906),
        _gige(label="40", hw_port='ge15', phy_addr=0x12, prim_offset=0x0907),

        _gige(label="41", hw_port='ge17', phy_addr=0x16, prim_offset=0x1101),
        _gige(label="42", hw_port='ge16', phy_addr=0x15, prim_offset=0x1100),
        _gige(label="43", hw_port='ge18', phy_addr=0x17, prim_offset=0x1102),
        _gige(label="44", hw_port='ge19', phy_addr=0x18, prim_offset=0x1103),
        _gige(label="45", hw_port='ge21', phy_addr=0x1a, prim_offset=0x1105),
        _gige(label="46", hw_port='ge20', phy_addr=0x19, prim_offset=0x1104),
        _gige(label="47", hw_port='ge22', phy_addr=0x1b, prim_offset=0x1106),
        _gige(label="48", hw_port='ge23', phy_addr=0x1c, prim_offset=0x1107),

        _sfp(label="49",  hw_port='hg0',  phy_addr=0x40, leds=(0, 0)),
        _sfp(label="50",  hw_port='hg1',  phy_addr=0x41, leds=(0, 2)),
        _sfp(label="51",  hw_port='hg2',  phy_addr=0x42, leds=(0, 4)),
        _sfp(label="52",  hw_port='hg3',  phy_addr=0x43, leds=(0, 6)),
    )

class LenovoNE0152TCPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/platform/lenovo_ne0152t_cpld.0'

# Intel CPU temp sensors

class LenovoNE0152TCoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 65
    temp_max = 85
    temp_crit = 98
            
class LenovoNE0152TCPUTemp1(LenovoNE0152TCoreTemp):
    name = 'Temp1'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class LenovoNE0152TCPUTemp2(LenovoNE0152TCoreTemp):
    name = 'Temp2'
    description = 'Core 1'
    driver_hwmon = ['temp3']

# Helix4 internal temp sensors

class LenovoNE0152THelix4Temp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_fan_min = 50
    temp_fan_max = 75
    temp_max     = 85
    temp_crit    = 105

class LenovoNE0152THelix4Temp1(LenovoNE0152THelix4Temp):
    name = 'Temp3'
    driver_hwmon = ['temp1']

class LenovoNE0152THelix4Temp2(LenovoNE0152THelix4Temp):
    name = 'Temp4'
    driver_hwmon = ['temp2']

class LenovoNE0152THelix4Temp3(LenovoNE0152THelix4Temp):
    name = 'Temp5'
    driver_hwmon = ['temp3']

# Board temp sensors

class LenovoNE0152TTemp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']
    temp_max = 70
    temp_fan_max = 78
    temp_crit = 85

class LenovoNE0152TTemp1(LenovoNE0152TTemp):
    driver_path = '/sys/bus/i2c/devices/14-0048'
    name = 'Temp6'
    description = 'Ambient 1'
    
class LenovoNE0152TTemp2(LenovoNE0152TTemp):
    driver_path = '/sys/bus/i2c/devices/15-0049'
    name = 'Temp7'
    description = 'Ambient 2'
    
class LenovoNE0152TTemp3(LenovoNE0152TTemp):
    driver_path = '/sys/bus/i2c/devices/16-004a'
    name = 'Temp8'
    description = 'Fan board'
    
class LenovoNE0152TTemp4(LenovoNE0152TTemp):
    driver_path = '/sys/bus/i2c/devices/17-004b'
    name = 'Temp9'
    description = 'CPU board'
    
# Fans

class LenovoNE0152TFan(cumulus.sensors.Fan_Unit):
    cpld_path = LenovoNE0152TCPLD.path
    driver_path = cpld_path
    fan_min = 0
    fan_max = 13000
    minpwm = 0

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.fan_present) == '1':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class LenovoNE0152TFan1(LenovoNE0152TFan):
    fan_present = 'fan1_present'
    name = 'Fan1'
    description = 'Fan Tray 1'
    driver_hwmon = ['fan1']
    pwm_hwmon = ['fan_pwm']

class LenovoNE0152TFan2(LenovoNE0152TFan):
    fan_present = 'fan2_present'
    name = 'Fan2'
    description = 'Fan Tray 2'
    driver_hwmon = ['fan2']
    pwm_hwmon = ['fan_pwm']

class LenovoNE0152TFan3(LenovoNE0152TFan):
    fan_present = 'fan3_present'
    name = 'Fan3'
    description = 'Fan Tray 3'
    driver_hwmon = ['fan3']
    pwm_hwmon = ['fan_pwm']

class LenovoNE0152TFan4(LenovoNE0152TFan):
    fan_present = 'fan1_present'
    name = 'Fan4'
    description = 'Fan Tray 1'
    driver_hwmon = ['fan4']
    pwm_hwmon = ['fan_pwm']

class LenovoNE0152TFan5(LenovoNE0152TFan):
    fan_present = 'fan2_present'
    name = 'Fan5'
    description = 'Fan Tray 2'
    driver_hwmon = ['fan5']
    pwm_hwmon = ['fan_pwm']

class LenovoNE0152TFan6(LenovoNE0152TFan):
    fan_present = 'fan3_present'
    name = 'Fan6'
    description = 'Fan Tray 3'
    driver_hwmon = ['fan6']
    pwm_hwmon = ['fan_pwm']

def LenovoNE0152T_Fan_Direction(fan):
    file_name = '/sys/devices/platform/lenovo_ne0152t_cpld.0/fan' + fan + '_direction'
    try:
        direction = subprocess.check_output(['cat',file_name]).rstrip()
    except:
        syslog.syslog(syslog.LOG_WARNING, "Could not determine fan " + fan + " direction.  Assuming 'F2B'")
        direction = 'F2B'
    return direction

# PSUs

class LenovoNE0152TPSU(cumulus.sensors.PSU_Unit):
    cpld_path = LenovoNE0152TCPLD.path

class LenovoNE0152TPSU1(LenovoNE0152TPSU):
    name = 'PSU1'
    description = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class LenovoNE0152TPSU2(LenovoNE0152TPSU):
    name = 'PSU2'
    description = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

# LEDs
'''
   Lenovo NE0152T LEDs

   System:
       off            - all systems OK
       blue blinking  - service required
   Stacking:
       off            - not used
   Power:
       green          - power OK
       green blinking - power supply failure
   Fan:
       green          - fans OK
       green blinking - fan failure
   3 x Rear Fan:
       green          - fans OK
       green blinking - fan failure
'''
class LenovoNE0152TLED(cumulus.sensors.StatusLED_Unit):
    led_path = LenovoNE0152TCPLD.path
    
class LenovoNE0152TSystemLED(LenovoNE0152TLED):
    name = 'System LED'
    led_location = 'led_system'
    good_led_color = 'off'
    bad_led_color = 'blue_blinking'

class LenovoNE0152TPowerLED(LenovoNE0152TLED):
    name = 'Power LED'
    sources = ['PSU1', 'PSU2']
    led_location = 'led_psu'
    bad_led_color = 'green_blinking'

class LenovoNE0152TFanLED(LenovoNE0152TLED):
    name = 'Fan LED'
    sources = ['Fan1', 'Fan2',
               'Fan3', 'Fan4',
               'Fan5', 'Fan6']
    led_location = 'led_fan'
    bad_led_color = 'green_blinking'

class LenovoNE0152TFan1LED(LenovoNE0152TLED):
    name = 'Fan 1 LED'
    sources = ['Fan1', 'Fan2']
    led_location = 'led_fan1'
    bad_led_color = 'green_blinking'

class LenovoNE0152TFan2LED(LenovoNE0152TLED):
    name = 'Fan 2 LED'
    sources = ['Fan3', 'Fan4']
    led_location = 'led_fan2'
    bad_led_color = 'green_blinking'

class LenovoNE0152TFan3LED(LenovoNE0152TLED):
    name = 'Fan 3 LED'
    sources = ['Fan5', 'Fan6']
    led_location = 'led_fan3'
    bad_led_color = 'green_blinking'

class LenovoNE0152TPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'

    if LenovoNE0152T_Fan_Direction('1') == 'B2F':
        fans = (LenovoNE0152TFan4(),
                )
    else:
        fans = (LenovoNE0152TFan1(),
                )
        
    if LenovoNE0152T_Fan_Direction('2') == 'B2F':
        fans = fans + (LenovoNE0152TFan5(),
                )
    else:
        fans = fans + (LenovoNE0152TFan2(),
                )
        
    if LenovoNE0152T_Fan_Direction('3') == 'B2F':
        fans = fans + (LenovoNE0152TFan6(),
                )
    else:
        fans = fans + (LenovoNE0152TFan3(),
                )

    temps = (LenovoNE0152TCPUTemp1(),
             LenovoNE0152TCPUTemp2(),
             LenovoNE0152THelix4Temp1(),
             LenovoNE0152THelix4Temp2(),
             LenovoNE0152THelix4Temp3(),
             LenovoNE0152TTemp1(),
             LenovoNE0152TTemp2(),
             LenovoNE0152TTemp3(),
             LenovoNE0152TTemp4(),
             )

# Units

class LenovoNE0152TPWMs():
    units = (LenovoNE0152TPWM1(),
             )

class LenovoNE0152TLEDs():
    units = (LenovoNE0152TSystemLED(),
             LenovoNE0152TPowerLED(),
             LenovoNE0152TFanLED(),
             LenovoNE0152TFan1LED(),
             LenovoNE0152TFan2LED(),
             LenovoNE0152TFan3LED(),
             )
    
class LenovoNE0152TSensors():
    units = (LenovoNE0152TCPUTemp1(),
             LenovoNE0152TCPUTemp2(),
             LenovoNE0152THelix4Temp1(),
             LenovoNE0152THelix4Temp2(),
             LenovoNE0152THelix4Temp3(),
             LenovoNE0152TTemp1(),
             LenovoNE0152TTemp2(),
             LenovoNE0152TTemp3(),
             LenovoNE0152TTemp4(),
             LenovoNE0152TPSU1(),
             LenovoNE0152TPSU2(),
             )

    if LenovoNE0152T_Fan_Direction('1') == 'B2F':
        units = units + (LenovoNE0152TFan4(),
                         )
    else:
        units = units + (LenovoNE0152TFan1(),
                         )

    if LenovoNE0152T_Fan_Direction('2') == 'B2F':
        units = units + (LenovoNE0152TFan5(),
                         )
    else:
        units = units + (LenovoNE0152TFan2(),
                         )

    if LenovoNE0152T_Fan_Direction('3') == 'B2F':
        units = units + (LenovoNE0152TFan6(),
                         )
    else:
        units = units + (LenovoNE0152TFan3(),
                         )

class LenovoNE0152T(cumulus.platform.Platform):
    name = 'lenovo,ne0152t'
    chip = cumulus.platform.Helix4Chip()
    switch = LenovoNE0152TSwitch(chip)
    cpld = LenovoNE0152TCPLD()
    statusleds = LenovoNE0152TLEDs()
    pwms = LenovoNE0152TPWMs()
    sensors = LenovoNE0152TSensors()

Platform = LenovoNE0152T
