#
# Copyright (c) 2018 Cumulus Networks, Inc.  All Rights Reserved.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class LenovoNE10032QSFP28(cumulus.platform.Tomahawk100GPort):
    '''
;;; Lenovo NE10032
;;;
;;; Each port has eight bits of LED data controlling four bi-color green/other
;;; LEDs.  Each LED behaves as follows:
;;;
;;;                 1st bit
;;;                0        1
;;;            +-------+-------+
;;;          0 | both  | other |
;;;  2nd bit   +-------+-------+
;;;          1 | green |  off  |
;;;            +-------+-------+
;;;'''
    _set_led_common = '''
;;;
;;; common LED routines
;;;
set_green_led:
; pack two bits
;  01 => green
;  11 => off
        ld      b, (ID_ENABLE_P)
        tst     b, ID_ENABLE_BIT
        jc      id_mode

        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack
        pushst  PORTSTATUS_ONE
        pack
	ret

set_led_off:
; pack two '1' bits
;  11 => off
        pushst PORTSTATUS_ONE
        pack
        pushst PORTSTATUS_ONE
        pack
        ret

id_mode:
        push    1
        cmp     a, (ID_PORT_P)
        jnz     pack_it
        ld      b, (WIGGLE_STATE_P)
        and     b, (WAG_STATE_P)
        push    b
        tinv
pack_it:
; pack two copies of T
        pop
        push    cy
        pack
        push    cy
        pack
        ret
'''
    _set_led_1x = '''
;;;
;;; In 1x mode, only the green LED of the first bi-color LED is used to
;;; indicate link/activity.
;;;
set_led:
        call    set_green_led  ; set the first LED to green/off
        call    set_led_off
        call    set_led_off
        call    set_led_off
        ret
'''
    _set_led_2x = '''
;;;
;;; In 2x mode, we use the green LED of the first and third bi-color LED to
;;; indicate link/activity. The rest are off.
;;;
set_led:
        call    set_green_led
        inc     a
        inc     a
        call    set_led_off
        call    set_green_led
        call    set_led_off
        ret
'''
    _set_led_4x = '''
;;;
;;; In 4x mode, the green LEDs of all four bi-color LEDs are used to
;;; indicate link/activity.
;;;
set_led:
        call    set_green_led
        inc     a
        call    set_green_led
        inc     a
        call    set_green_led
        inc     a
        call    set_green_led
        ret
'''

    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None

        return set_calls

class LenovoNE10032LED(cumulus.ledup.TomahawkLEDController):
    led_functions = (LenovoNE10032QSFP28._set_led_common)
    num_leds = 128
    has_phys_id = True


# Switch Ports

class LenovoNE10032Switch(cumulus.platform.BCMSwitch):
    ledup = LenovoNE10032LED
    _qsfp28 = LenovoNE10032QSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp28(label="1",  serdes=( 32,  33,  34,  35), leds=(1,   0), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="2",  serdes=( 36,  37,  38,  39), leds=(1,   8), tx_lane_map=0x3210, rx_lane_map=0x1230, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="3",  serdes=( 40,  41,  42,  43), leds=(1,  16), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="4",  serdes=( 44,  45,  46,  47), leds=(1,  24), tx_lane_map=0x3210, rx_lane_map=0x1230, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),

        _qsfp28(label="5",  serdes=( 48,  49,  50,  51), leds=(1,  32), tx_lane_map=0x0123, rx_lane_map=0x1230, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="6",  serdes=( 52,  53,  54,  55), leds=(1,  40), tx_lane_map=0x3210, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="7",  serdes=( 56,  57,  58,  59), leds=(1,  48), tx_lane_map=0x0123, rx_lane_map=0x1230, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="8",  serdes=( 60,  61,  62,  63), leds=(1,  56), tx_lane_map=0x3210, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),

        _qsfp28(label="9",  serdes=( 64,  65,  66,  67), leds=(1,  64), tx_lane_map=0x2301, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="10", serdes=( 68,  69,  70,  71), leds=(1,  72), tx_lane_map=0x3210, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="11", serdes=( 72,  73,  74,  75), leds=(1,  80), tx_lane_map=0x0123, rx_lane_map=0x1230, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="12", serdes=( 76,  77,  78,  79), leds=(1,  88), tx_lane_map=0x1032, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),

        _qsfp28(label="13", serdes=( 80,  81,  82,  83), leds=(1,  96), tx_lane_map=0x0123, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="14", serdes=( 84,  85,  86,  87), leds=(1, 104), tx_lane_map=0x1032, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="15", serdes=( 88,  89,  90,  91), leds=(1, 112), tx_lane_map=0x0123, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="16", serdes=( 92,  93,  94,  95), leds=(1, 120), tx_lane_map=0x1032, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),

        _qsfp28(label="17", serdes=( 96,  97,  98,  99), leds=(0,   0), tx_lane_map=0x1032, rx_lane_map=0x3210, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="18", serdes=(100, 101, 102, 103), leds=(0,   8), tx_lane_map=0x1032, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="19", serdes=(104, 105, 106, 107), leds=(0,  16), tx_lane_map=0x1032, rx_lane_map=0x3210, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="20", serdes=(108, 109, 110, 111), leds=(0,  24), tx_lane_map=0x1032, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),

        _qsfp28(label="21", serdes=(112, 113, 114, 115), leds=(0,  32), tx_lane_map=0x1032, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="22", serdes=(116, 117, 118, 119), leds=(0,  40), tx_lane_map=0x1032, rx_lane_map=0x1230, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="23", serdes=(120, 121, 122, 123), leds=(0,  48), tx_lane_map=0x3210, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="24", serdes=(124, 125, 126, 127), leds=(0,  56), tx_lane_map=0x3210, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),

        _qsfp28(label="25", serdes=(  0,   1,   2,   3), leds=(0,  64), tx_lane_map=0x3210, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="26", serdes=(  4,   5,   6,   7), leds=(0,  72), tx_lane_map=0x1032, rx_lane_map=0x1230, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="27", serdes=(  8,   9,  10,  11), leds=(0,  80), tx_lane_map=0x3210, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="28", serdes=( 12,  13,  14,  15), leds=(0,  88), tx_lane_map=0x1032, rx_lane_map=0x1230, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),

        _qsfp28(label="29", serdes=( 16,  17,  18,  19), leds=(0,  96), tx_lane_map=0x3210, rx_lane_map=0x1230, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="30", serdes=( 20,  21,  22,  23), leds=(0, 104), tx_lane_map=0x3210, rx_lane_map=0x1230, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="31", serdes=( 24,  25,  26,  27), leds=(0, 112), tx_lane_map=0x3210, rx_lane_map=0x1230, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="32", serdes=( 28,  29,  30,  31), leds=(0, 120), tx_lane_map=0x1230, rx_lane_map=0x1230, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
    )

class LenovoNE10032CPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/platform/lenovo_ne10032_cpld.0'

# Intel CPU temp sensors

class LenovoNE10032CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 65
    temp_max = 85
    temp_crit = 98
            
class LenovoNE10032CPUTemp1(LenovoNE10032CoreTemp):
    name = 'Temp1'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class LenovoNE10032CPUTemp2(LenovoNE10032CoreTemp):
    name = 'Temp2'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class LenovoNE10032CPUTemp3(LenovoNE10032CoreTemp):
    name = 'Temp3'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class LenovoNE10032CPUTemp4(LenovoNE10032CoreTemp):
    name = 'Temp4'
    description = 'Core 3'
    driver_hwmon = ['temp5']

# Tomahawk internal temp sensors

class LenovoNE10032TomahawkTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 105
    temp_crit    = 110

class LenovoNE10032TomahawkTemp1(LenovoNE10032TomahawkTemp):
    name = 'Temp5'
    driver_hwmon = ['temp1']

class LenovoNE10032TomahawkTemp2(LenovoNE10032TomahawkTemp):
    name = 'Temp6'
    driver_hwmon = ['temp2']

class LenovoNE10032TomahawkTemp3(LenovoNE10032TomahawkTemp):
    name = 'Temp7'
    driver_hwmon = ['temp3']

class LenovoNE10032TomahawkTemp4(LenovoNE10032TomahawkTemp):
    name = 'Temp8'
    driver_hwmon = ['temp4']

class LenovoNE10032TomahawkTemp5(LenovoNE10032TomahawkTemp):
    name = 'Temp9'
    driver_hwmon = ['temp5']

class LenovoNE10032TomahawkTemp6(LenovoNE10032TomahawkTemp):
    name = 'Temp10'
    driver_hwmon = ['temp6']

class LenovoNE10032TomahawkTemp7(LenovoNE10032TomahawkTemp):
    name = 'Temp11'
    driver_hwmon = ['temp7']

class LenovoNE10032TomahawkTemp8(LenovoNE10032TomahawkTemp):
    name = 'Temp12'
    driver_hwmon = ['temp8']

# Board temp sensors

class LenovoNE10032Temp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']
    temp_max = 61
    temp_fan_max = 68
    temp_crit = 75
    
class LenovoNE10032Temp1(LenovoNE10032Temp):
    driver_path = '/sys/bus/i2c/devices/0-004f'
    name = 'Temp13'
    description = 'CPU board'
    
class LenovoNE10032Temp2(LenovoNE10032Temp):
    driver_path = '/sys/bus/i2c/devices/12-004d'
    name = 'Temp14'
    description = 'Hot spot'
    
class LenovoNE10032Temp3(LenovoNE10032Temp):
    driver_path = '/sys/bus/i2c/devices/13-004c'
    name = 'Temp15'
    description = 'Ambient'
    
# Fans

class LenovoNE10032FanBase(cumulus.sensors.Fan_Unit):
    cpld_path = LenovoNE10032CPLD.path
    driver_path = cpld_path
    minpwm = 0

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.fan_present) == '1':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class LenovoNE10032Fan_0(LenovoNE10032FanBase):
    fan_min = 0
    fan_max = 19000
    
class LenovoNE10032Fan_1(LenovoNE10032FanBase):
    fan_min = 0
    fan_max = 21000
    
class LenovoNE10032Fan1(LenovoNE10032Fan_0):
    fan_present = 'fantray1_present'
    name = 'Fan1_0'
    description = 'Fan1_0'
    driver_hwmon = ['fan1']
    pwm_hwmon = ['fan_pwm']

class LenovoNE10032Fan2(LenovoNE10032Fan_1):
    fan_present = 'fantray1_present'
    name = 'Fan1_1'
    description = 'Fan1_1'
    driver_hwmon = ['fan2']
    pwm_hwmon = ['fan_pwm']

class LenovoNE10032Fan3(LenovoNE10032Fan_0):
    fan_present = 'fantray2_present'
    name = 'Fan2_0'
    description = 'Fan2_0'
    driver_hwmon = ['fan3']
    pwm_hwmon = ['fan_pwm']

class LenovoNE10032Fan4(LenovoNE10032Fan_1):
    fan_present = 'fantray2_present'
    name = 'Fan2_1'
    description = 'Fan2_1'
    driver_hwmon = ['fan4']
    pwm_hwmon = ['fan_pwm']

class LenovoNE10032Fan5(LenovoNE10032Fan_0):
    fan_present = 'fantray3_present'
    name = 'Fan3_0'
    description = 'Fan3_0'
    driver_hwmon = ['fan5']
    pwm_hwmon = ['fan_pwm']

class LenovoNE10032Fan6(LenovoNE10032Fan_1):
    fan_present = 'fantray3_present'
    name = 'Fan3_1'
    description = 'Fan3_1'
    driver_hwmon = ['fan6']
    pwm_hwmon = ['fan_pwm']

class LenovoNE10032Fan7(LenovoNE10032Fan_0):
    fan_present = 'fantray4_present'
    name = 'Fan4_0'
    description = 'Fan4_0'
    driver_hwmon = ['fan7']
    pwm_hwmon = ['fan_pwm']

class LenovoNE10032Fan8(LenovoNE10032Fan_1):
    fan_present = 'fantray4_present'
    name = 'Fan4_1'
    description = 'Fan4_1'
    driver_hwmon = ['fan8']
    pwm_hwmon = ['fan_pwm']

class LenovoNE10032Fan9(LenovoNE10032Fan_0):
    fan_present = 'fantray5_present'
    name = 'Fan5_0'
    description = 'Fan5_0'
    driver_hwmon = ['fan9']
    pwm_hwmon = ['fan_pwm']

class LenovoNE10032Fan10(LenovoNE10032Fan_1):
    fan_present = 'fantray5_present'
    name = 'Fan5_1'
    description = 'Fan5_1'
    driver_hwmon = ['fan10']
    pwm_hwmon = ['fan_pwm']

class LenovoNE10032Fan11(LenovoNE10032Fan_0):
    fan_present = 'fantray6_present'
    name = 'Fan6_0'
    description = 'Fan6_0'
    driver_hwmon = ['fan11']
    pwm_hwmon = ['fan_pwm']

class LenovoNE10032Fan12(LenovoNE10032Fan_1):
    fan_present = 'fantray6_present'
    name = 'Fan6_1'
    description = 'Fan6_1'
    driver_hwmon = ['fan12']
    pwm_hwmon = ['fan_pwm']

# PSUs

class LenovoNE10032PSU(cumulus.sensors.PSU_Unit):
    cpld_path = LenovoNE10032CPLD.path

class LenovoNE10032PSU1(LenovoNE10032PSU):
    name = 'PSU1'
    description = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class LenovoNE10032PSU2(LenovoNE10032PSU):
    name = 'PSU2'
    description = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

# LEDs
'''
   Lenovo NE10032 LEDs

   System:
       off            - all systems OK
       blue blinking  - service required
   Stacking:
       off            - not used
   Power:
       green          - power OK
       green blinking - power supply failure
   6 x Fan:
       green          - fans OK
       green blinking - fan failure
   6 x Rear Fan:
       green          - fans OK
       green blinking - fan failure
'''
class LenovoNE10032LED(cumulus.sensors.StatusLED_Unit):
    led_path = LenovoNE10032CPLD.path
    
class LenovoNE10032SystemLED(LenovoNE10032LED):
    name = 'System LED'
    led_location = 'led_system'
    good_led_color = 'off'
    bad_led_color = 'blue_blinking'

class LenovoNE10032PowerLED(LenovoNE10032LED):
    name = 'Power LED'
    sources = ['PSU1', 'PSU2']
    led_location = 'led_psu'
    bad_led_color = 'green_blinking'

class LenovoNE10032FanTray1LED(LenovoNE10032LED):
    name = 'Fan Tray 1 LED'
    sources = ['Fan1_0', 'Fan1_1']
    led_location = 'led_fan1'
    bad_led_color = 'green_blinking'

class LenovoNE10032FanTray2LED(LenovoNE10032LED):
    name = 'Fan Tray 2 LED'
    sources = ['Fan2_0', 'Fan2_1']
    led_location = 'led_fan2'
    bad_led_color = 'green_blinking'

class LenovoNE10032FanTray3LED(LenovoNE10032LED):
    name = 'Fan Tray 3 LED'
    sources = ['Fan3_0', 'Fan3_1']
    led_location = 'led_fan3'
    bad_led_color = 'green_blinking'

class LenovoNE10032FanTray4LED(LenovoNE10032LED):
    name = 'Fan Tray 4 LED'
    sources = ['Fan4_0', 'Fan4_1']
    led_location = 'led_fan4'
    bad_led_color = 'green_blinking'

class LenovoNE10032FanTray5LED(LenovoNE10032LED):
    name = 'Fan Tray 5 LED'
    sources = ['Fan5_0', 'Fan5_1']
    led_location = 'led_fan5'
    bad_led_color = 'green_blinking'

class LenovoNE10032FanTray6LED(LenovoNE10032LED):
    name = 'Fan Tray 6 LED'
    sources = ['Fan6_0', 'Fan6_1']
    led_location = 'led_fan6'
    bad_led_color = 'green_blinking'

class LenovoNE10032FanTray1RearLED(LenovoNE10032LED):
    name = 'Fan Tray 1 Rear LED'
    sources = ['Fan1_0', 'Fan1_1']
    led_location = 'led_fan1_rear'
    bad_led_color = 'green_blinking'

class LenovoNE10032FanTray2RearLED(LenovoNE10032LED):
    name = 'Fan Tray 2 Rear LED'
    sources = ['Fan2_0', 'Fan2_1']
    led_location = 'led_fan2_rear'
    bad_led_color = 'green_blinking'

class LenovoNE10032FanTray3RearLED(LenovoNE10032LED):
    name = 'Fan Tray 3 Rear LED'
    sources = ['Fan3_0', 'Fan3_1']
    led_location = 'led_fan3_rear'
    bad_led_color = 'green_blinking'

class LenovoNE10032FanTray4RearLED(LenovoNE10032LED):
    name = 'Fan Tray 4 Rear LED'
    sources = ['Fan4_0', 'Fan4_1']
    led_location = 'led_fan4_rear'
    bad_led_color = 'green_blinking'

class LenovoNE10032FanTray5RearLED(LenovoNE10032LED):
    name = 'Fan Tray 5 Rear LED'
    sources = ['Fan5_0', 'Fan5_1']
    led_location = 'led_fan5_rear'
    bad_led_color = 'green_blinking'

class LenovoNE10032FanTray6RearLED(LenovoNE10032LED):
    name = 'Fan Tray 6 Rear LED'
    sources = ['Fan6_0', 'Fan6_1']
    led_location = 'led_fan6_rear'
    bad_led_color = 'green_blinking'

class LenovoNE10032PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'

    fans = (LenovoNE10032Fan1(),
            LenovoNE10032Fan2(),
            LenovoNE10032Fan3(),
            LenovoNE10032Fan4(),
            LenovoNE10032Fan5(),
            LenovoNE10032Fan6(),
            LenovoNE10032Fan7(),
            LenovoNE10032Fan8(),
            LenovoNE10032Fan9(),
            LenovoNE10032Fan10(),
            LenovoNE10032Fan11(),
            LenovoNE10032Fan12(),
            )

    temps = (LenovoNE10032CPUTemp1(),
             LenovoNE10032CPUTemp2(),
             LenovoNE10032CPUTemp3(),
             LenovoNE10032CPUTemp4(),
             LenovoNE10032TomahawkTemp1(),
             LenovoNE10032TomahawkTemp2(),
             LenovoNE10032TomahawkTemp3(),
             LenovoNE10032TomahawkTemp4(),
             LenovoNE10032TomahawkTemp5(),
             LenovoNE10032TomahawkTemp6(),
             LenovoNE10032TomahawkTemp7(),
             LenovoNE10032TomahawkTemp8(),
             LenovoNE10032Temp1(),
             LenovoNE10032Temp2(),
             LenovoNE10032Temp3(),
             )

# Units

class LenovoNE10032PWMs():
    units = (LenovoNE10032PWM1(),
             )

class LenovoNE10032LEDs():
    units = (LenovoNE10032SystemLED(),
             LenovoNE10032PowerLED(),
             LenovoNE10032FanTray1LED(),
             LenovoNE10032FanTray2LED(),
             LenovoNE10032FanTray3LED(),
             LenovoNE10032FanTray4LED(),
             LenovoNE10032FanTray5LED(),
             LenovoNE10032FanTray6LED(),
             LenovoNE10032FanTray1RearLED(),
             LenovoNE10032FanTray2RearLED(),
             LenovoNE10032FanTray3RearLED(),
             LenovoNE10032FanTray4RearLED(),
             LenovoNE10032FanTray5RearLED(),
             LenovoNE10032FanTray6RearLED(),
             )
    
class LenovoNE10032Sensors():
    units = (LenovoNE10032CPUTemp1(),
             LenovoNE10032CPUTemp2(),
             LenovoNE10032CPUTemp3(),
             LenovoNE10032CPUTemp4(),
             LenovoNE10032TomahawkTemp1(),
             LenovoNE10032TomahawkTemp2(),
             LenovoNE10032TomahawkTemp3(),
             LenovoNE10032TomahawkTemp4(),
             LenovoNE10032TomahawkTemp5(),
             LenovoNE10032TomahawkTemp6(),
             LenovoNE10032TomahawkTemp7(),
             LenovoNE10032TomahawkTemp8(),
             LenovoNE10032Temp1(),
             LenovoNE10032Temp2(),
             LenovoNE10032Temp3(),
             LenovoNE10032Fan1(),
             LenovoNE10032Fan2(),
             LenovoNE10032Fan3(),
             LenovoNE10032Fan4(),
             LenovoNE10032Fan5(),
             LenovoNE10032Fan6(),
             LenovoNE10032Fan7(),
             LenovoNE10032Fan8(),
             LenovoNE10032Fan9(),
             LenovoNE10032Fan10(),
             LenovoNE10032Fan11(),
             LenovoNE10032Fan12(),
             LenovoNE10032PSU1(),
             LenovoNE10032PSU2(),
             )

class LenovoNE10032(cumulus.platform.Platform):
    name = 'lenovo,ne10032'
    chip = cumulus.platform.TomahawkChip()
    switch = LenovoNE10032Switch(chip)
    cpld = LenovoNE10032CPLD()
    statusleds = LenovoNE10032LEDs()
    pwms = LenovoNE10032PWMs()
    sensors = LenovoNE10032Sensors()

Platform = LenovoNE10032
