#!/usr/bin/python
#
# Copyright 2013.  Cumulus Networks, Inc.
#
# porttab.py --
#
#    parse a porttab file and provide lookup methods.
#
import errno
try:
    import bcmshell
    bcmShellAvailable = True
except ImportError:
    bcmShellAvailable = False

class NoSuchLinuxInterface(Exception):
    pass
class NoSuchBCMInterface(Exception):
    pass
class NoSuchLogicalPort(Exception):
    pass
class NotConfigured(Exception):
    pass
class UnsupportedPlatformError(RuntimeError):
    pass
class NoSuchLogicalPort(RuntimeError):
    pass
class NoSuchDPort(RuntimeError):
    pass
class BCMShellNotAvailable(RuntimeError):
    pass
class BCMShellError(RuntimeError):
    pass

def safe_list_get(the_list, idx, default=''):
    try:
        return the_list[idx]
    except IndexError:
        pass
    return default

class bcmPortmapEntry:
    def __init__(self, dport, logical_port):
        self.dport = dport
        self.logical_port = logical_port

class porttabEntry:
    def __init__(self, columns):
        self.linux_intf = safe_list_get(columns, 0)
        self.bcm = safe_list_get(columns, 1)
        self.unit = safe_list_get(columns, 2)
        self.is_fabric = safe_list_get(columns, 3)
        self.logical_port = int(safe_list_get(columns, 4, '-1'))
        self.is_loopback = bool(int(safe_list_get(columns, 5, '0')))

class porttab:
    def __init__(self, path='/var/lib/cumulus/porttab', use_hw=True):
        self._readpt(path)
        self._read_hw_portmap(use_hw)

    def _readpt(self, path):
        self.porttab = []
        self.use_logical = True

        try:
            ptfile = open(path, 'r')
        except IOError, e:
            if e.errno == errno.ENOENT:
                raise NotConfigured('porttab not configured')
            else:
                raise e

        for line in ptfile.readlines():
            line = line.strip()

            if '#' in line:
                continue

            try:
                pe = porttabEntry(line.split())
            except ValueError:
                continue

            if pe.logical_port == -1:
                self.use_logical = False

            self.porttab.append(pe)

    def _read_hw_portmap(self, use_hw):
        self.bcm_portmap = []

        if not self.use_logical or not use_hw:
            return

        if not bcmShellAvailable:
            raise BCMShellNotAvailable('show portmap is not supported on this platform')

        # Sample "show portmap" output (T2) -
        #
        #             logical  physical    mmu   ucast Qbase  mcast Qbase
        #      cpu0      0         0        52         0         2000
        #       xe0      1        25         6        72         1540
        # >>>>>>>>>>>>>>>>>>>>>>>>>> SNIP >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
        try:
            portmap_str = bcmshell.bcmshell().run('show portmap')
        except IOError:
            # bcmshell is not accessible, maybe switchd is not running?
            raise BCMShellError('show portmap timed out')

        portmap_lines = portmap_str.splitlines()
        if len(portmap_lines) < 2 or 'Error' in portmap_lines[0]:
            raise UnsupportedPlatformError('show portmap is not supported on this chip')

        # find the position of the logical port column - is different
        # on TH (2) and T2 (1)
        try:
            logicalIndex = portmap_lines[0].split().index('logical') + 1
        except ValueError:
            raise UnsupportedPlatformError('show portmap format unexpected')

        for port_str in portmap_lines[1:]:
            port_cols = port_str.split()
            portmap_entry = bcmPortmapEntry(port_cols[0], int(port_cols[logicalIndex]))
            self.bcm_portmap.append(portmap_entry)

    def logical_ports_available(self):
        return True if self.bcm_portmap else False

    def dport2logical(self, dport):
        for portmap_entry in self.bcm_portmap:
            if portmap_entry.dport == dport:
                return portmap_entry.logical_port
        raise NoSuchDPort(dport)

    def logical2dport(self, logical_port):
        for portmap_entry in self.bcm_portmap:
            if portmap_entry.logical_port == logical_port:
                return portmap_entry.dport
        raise NoSuchLogicalPort(logical_port)

    def linux2bcm(self, linux_intf):
        for pe in self.porttab:
            if pe.linux_intf == linux_intf:
                if self.bcm_portmap:
                    return self.logical2dport(pe.logical_port)
                return pe.bcm

        raise NoSuchLinuxInterface(linux_intf)

    def _bcm2linux_from_portmap(self, bcm):
        logical_port  = self.dport2logical(bcm)

        for pe in self.porttab:
            if pe.logical_port == logical_port:
                return pe.linux_intf

        raise NoSuchBCMInterface(bcm)

    def bcm2linux(self, bcm):
        if self.bcm_portmap:
            return self._bcm2linux_from_portmap(bcm)

        for pe in self.porttab:
            if pe.bcm == bcm:
                return pe.linux_intf

        raise NoSuchBCMInterface(bcm)

    def linux2mlx(self, linux_intf):
        for pe in self.porttab:
            if pe.linux_intf == linux_intf:
                return ((int(pe.unit)+1) << 16) + (int(pe.bcm[2:]) << 8)

        raise NoSuchLinuxInterface(linux_intf)

    def linux2logical(self, linux_intf):
        for pe in self.porttab:
            if pe.linux_intf == linux_intf:
                return pe.logical_port
        raise NoSuchLinuxInterface

    def logical2linux(self, logical_port):
        for pe in self.porttab:
            if pe.logical_port == logical_port:
                return pe.linux_intf
        raise NoSuchLogicalPort

    def bcm2logical(self, bcm):
        for pe in self.porttab:
            if pe.bcm == bcm:
                return pe.logical_port
        raise NoSuchBCMInterface

    def logical2bcm(self, logical_port):
        for pe in self.porttab:
            if pe.logical_port == logical_port:
                return pe.bcm
        raise NoSuchLogicalPort

    def get_linux_ports(self):
        return [p.linux_intf for p in self.porttab]

    def get_bcm_ports(self):
        if self.bcm_portmap:
            return [self.logical2dport(p.logical_port) for p in self.porttab]
        return [p.bcm for p in self.porttab]


